import logging

from bitfield import BitField, BitHandler
from django.conf import settings
from django.db import transaction
from django.db.models import TextField, F
from django.forms import model_to_dict

from isc_common import delAttr, setAttr, getAttr
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn, TurnBitOn
from isc_common.common import getOrElse
from isc_common.datetime import DateToStr
from isc_common.datetime import StrToDate
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.ws.progressStack import ProgressStack
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.production.models.operation_material import Operation_material
from kaf_pas.production.models.operation_resources import Operation_resources
from kaf_pas.production.models.operations_item import Operations_item
from kaf_pas.sales.models.demand import Demand

logger = logging.getLogger(__name__)


class Ready_2_launchQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Ready_2_launchManager(AuditManager):

    @staticmethod
    def make(demand, user, ready_2_launch=None, props=32):
        from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail

        if isinstance(demand, int):
            demand = Demand.objects.get(id=demand)
        elif not isinstance(demand, Demand):
            raise Exception(f'demand must be a Demand instance or int')

        if isinstance(user, int):
            user = User.objects.get(id=user)
        elif not isinstance(user, User):
            raise Exception(f'user must be a User instance or int')

        cnt = 0
        cnt_not = 0

        all_notes = []
        options = []

        if IsBitOn(props, 0):
            options.append('Включена опция проверки наличия у операции длительности выполнения.')

        if IsBitOn(props, 1):
            options.append('Включена опция проверки наличия у операции № п/п.')

        if IsBitOn(props, 2):
            options.append('Включена опция проверки наличия у операции материалов или стандартных изделий.')

        if IsBitOn(props, 3):
            options.append('Включена опция проверки наличия у операции ресурса либо места выполнения.')

        if IsBitOn(props, 4):
            options.append('Включена опция проверки наличия у операции единицы измерения.')

        if IsBitOn(props, 5):
            options.append('Включена опция проверки наличия операций.')

        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
        )

        id = f'demand_{demand.id}_{ready_2_launch.props}_{user.id}'
        try:
            demand_str = f'<h3>Оценка готовности к запуску: Заказ № {demand.code} от {DateToStr(demand.date)}</h3>'
            progress.show(
                title=f'<b>Обработано товарных позиций</b>',
                label_contents=demand_str,
                cntAll=Item_refs.objects.get_descendants_count(id=demand.precent_item.item.id),
                id=id
            )

            if not ready_2_launch:
                ready_2_launch, _ = Ready_2_launch.objects.get_or_create(demand=demand)

            Item_refs.objects.filter(child=F('parent')).delete()
            for item_ref in Item_refs.objects.get_descendants(id=demand.precent_item.item.id):
                notes = []
                operations_cnt = Operations_item.objects.filter(item=item_ref.child).count()
                STMP_1 = item_ref.child.STMP_1.value_str if item_ref.child.STMP_1 else None
                STMP_2 = item_ref.child.STMP_2.value_str if item_ref.child.STMP_2 else None
                section = None
                try:
                    section = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child).section
                except Item_line.DoesNotExist:
                    # cnt_not += 1
                    notes.append(f'Наименование: {getOrElse("", STMP_1)}, Обозначение: {getOrElse("", STMP_2)} ID={item_ref.child.id}, PARENT_ID={item_ref.parent.id} не входит в детализацию.')

                cnt_not1 = 0
                if section and section != 'Документация':
                    if operations_cnt == 0:
                        if IsBitOn(props, 5):
                            cnt_not += 1
                            notes.append(f'Наименование: {getOrElse("", STMP_1)}, Обозначение: {getOrElse("", STMP_2)} ID={item_ref.child.id}, PARENT_ID={item_ref.parent.id} не указаны операции.')
                    else:
                        for operation in Operations_item.objects.filter(item=item_ref.child).exclude(operation__code='noop'):
                            if IsBitOn(props, 0) and not operation.qty:
                                cnt_not1 = 1
                                notes.append(f'Операция: {operation.operation.full_name} не указана длительность.')

                            if IsBitOn(props, 1) and not operation.num:
                                cnt_not1 = 1
                                notes.append(f'Операция: {operation.operation.full_name} не указан № п/п.')

                            if IsBitOn(props, 2):
                                operation_material_cnt = Operation_material.objects.filter(operationitem=operation).count()
                                if operation_material_cnt == 0:
                                    cnt_not1 = 1
                                    notes.append(f'Операция: {operation.operation.full_name} не указаны материалы или стандартные изделия.')

                            if IsBitOn(props, 3):
                                operation_resources_cnt = Operation_resources.objects.filter(operationitem=operation).count()
                                if operation_resources_cnt == 0:
                                    cnt_not1 = 1
                                    notes.append(f'Операция: {operation.operation.full_name} не указан ресурс либо место выполнения.')

                            if IsBitOn(props, 4) and not operation.ed_izm:
                                cnt_not1 = 1
                                notes.append(f'Операция: {operation.operation.full_name} не указана единица измерения.')

                if len(notes) > 0:
                    notes_str = "\n".join(notes)
                    Ready_2_launch_detail.objects.get_or_create(
                        item=item_ref.child,
                        ready=ready_2_launch,
                        notes=notes_str
                        # notes=f'<pre>{notes_str}</pre>'
                    )
                cnt_not += cnt_not1
                progress.setCntDone(cnt, id)
                cnt += 1

            ready = round(100 - cnt_not / cnt * 100, 2)
            all_notes.append(f'Готовность: {ready}%')
            notes_str = "\n".join(all_notes)
            ready_2_launch.notes = f'<pre>{notes_str}</pre>'
            ready_2_launch.save()
            progress.close(id=id)

            options_str = "\n" + "\n".join(options)
            settings.EVENT_STACK.EVENTS_PRODUCTION_READY_2_LAUNCH.send_message(f'Выполнена {demand_str} <h3>готовность: {ready} </h3>{options_str}<p/>')

        except Exception as ex:
            progress.close(id=id)
            raise ex

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        demand_id = _data.get('demand_id')
        delAttr(_data, 'demand__date')
        delAttr(_data, 'demand__code')
        delAttr(_data, 'demand__description')
        delAttr(_data, 'demand__precent_item__STMP_1__value_str')
        delAttr(_data, 'demand__precent_item__STMP_2__value_str')

        props = 0
        if _data.get('check_qty'):
            props = TurnBitOn(props, 0)
        delAttr(_data, 'check_qty')

        if _data.get('check_num'):
            props = TurnBitOn(props, 1)
        delAttr(_data, 'check_num')

        if _data.get('check_material'):
            props = TurnBitOn(props, 2)
        delAttr(_data, 'check_material')

        if _data.get('check_resources'):
            props = TurnBitOn(props, 3)
        delAttr(_data, 'check_resources')

        if _data.get('check_edizm'):
            props = TurnBitOn(props, 4)
        delAttr(_data, 'check_edizm')

        if _data.get('check_operation'):
            props = TurnBitOn(props, 5)
        delAttr(_data, 'check_operation')

        setAttr(_data, 'props', props)

        with transaction.atomic():
            date = StrToDate(_data.get('date'))
            setAttr(_data, 'lastmodified', date)
            delAttr(_data, 'date')
            res = super().create(**_data)
            Ready_2_launchManager.make(
                demand=demand_id,
                user=request.user_id,
                ready_2_launch=res, props=props
            )

            res = model_to_dict(res)
            props = getAttr(res, 'props')
            if isinstance(props, BitHandler):
                props = getAttr(res, 'props')._value
                setAttr(res, 'props', props)
            data.update(res)
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        return data

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'date': record.lastmodified,
            'demand_id': record.demand.id,
            'demand__date': record.demand.date,
            'demand__code': record.demand.code,
            'demand__name': record.demand.name,
            'check_qty': IsBitOn(record.props, 0),
            'check_num': IsBitOn(record.props, 1),
            'check_material': IsBitOn(record.props, 2),
            'check_resources': IsBitOn(record.props, 3),
            'check_edizm': IsBitOn(record.props, 4),
            'check_operation': IsBitOn(record.props, 5),
            'demand__description': record.demand.description,
            'demand__precent_item_id': record.demand.precent_item.id,
            'demand__precent_item__STMP_1_id': record.demand.precent_item.item.STMP_1.id if record.demand.precent_item.item.STMP_1 else None,
            'demand__precent_item__STMP_1__value_str': record.demand.precent_item.item.STMP_1.value_str if record.demand.precent_item.item.STMP_1 else None,
            'demand__precent_item__STMP_2_id': record.demand.precent_item.item.STMP_2.id if record.demand.precent_item.item.STMP_2 else None,
            'demand__precent_item__STMP_2__value_str': record.demand.precent_item.item.STMP_2.value_str if record.demand.precent_item.item.STMP_2 else None,
            'notes': record.notes,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Ready_2_launchQuerySet(self.model, using=self._db)


class Ready_2_launch(AuditModel):
    demand = ForeignKeyCascade(Demand)
    notes = TextField(null=True, blank=True)
    props = BitField(flags=(
        ('check_qty', 'Проверять длительность'),  # 1
        ('check_num', 'Проверять № п/п'),  # 2
        ('check_material', 'Проверять материалы'),  # 4
        ('check_resources', 'Проверять ресурсы'),  # 8
        ('check_edizm', 'Проверять еденицу измерения'),  # 16
        ('check_operation', 'Проверять операцию'),  # 32
    ), default=0, db_index=True)

    objects = Ready_2_launchManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Готовность к запуску'



import logging

from bitfield import BitField
from django.db import transaction
from django.db.models import DecimalField
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.bit import IsBitOn
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade, ForeignKeySetNull
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.number import DelProps
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.material_askon import Material_askon
from kaf_pas.ckk.models.materials import Materials
from kaf_pas.production.models.launch_operations_item import Launch_operations_item
from kaf_pas.production.models.operation_material import Operation_material

logger = logging.getLogger(__name__)


class Launch_operations_materialQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        if kwargs.get('material'):
            setAttr(kwargs, 'material_id', kwargs.get('material').id)
            delAttr(kwargs, 'material')

        if kwargs.get('material_askon'):
            setAttr(kwargs, 'material_askon_id', kwargs.get('material_askon').id)
            delAttr(kwargs, 'material_askon')

        if not kwargs.get('material_id') and not kwargs.get('material_askon_id'):
            raise Exception('Необходим хотябы один выпбранный параметр.')
        setAttr(kwargs, '_operation', 'create')
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Launch_operations_materialManager(CommonManagetWithLookUpFieldsManager):

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        launch_operationitem_id = _data.get('launch_operationitem_id')

        res = []
        if isinstance(launch_operationitem_id, list):
            delAttr(_data, 'launch_operationitem_id')
            with transaction.atomic():
                for launch_operationitem in launch_operationitem_id:
                    setAttr(_data, 'launch_operationitem_id', launch_operationitem)
                    operation_material, created = super().get_or_create(**_data)
                    if created:
                        _res = model_to_dict(operation_material)
                        setAttr(_res, 'material_askon__field0', operation_material.material_askon.field0 if operation_material.material_askon else None)
                        setAttr(_res, 'complex_name', operation_material.complex_name)
                        setAttr(_res, 'complex_gost', operation_material.complex_gost)
                        setAttr(_res, 'edizm__code', operation_material.edizm.code)
                        res.append(DelProps(_res))
        else:
            operation_material, created = super().get_or_create(**_data)
            if created:
                _res = model_to_dict(operation_material)
                setAttr(_res, 'material_askon__field0', operation_material.material_askon.field0 if operation_material.material_askon else None)
                setAttr(_res, 'complex_name', operation_material.complex_name)
                setAttr(_res, 'complex_gost', operation_material.complex_gost)
                setAttr(_res, 'edizm__code', operation_material.edizm.code)
                res.append(DelProps(_res))
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_material_id': record.operation_material.id if record.operation_material else None,
            'launch_operationitem_id': record.launch_operationitem.id,
            'material_askon_id': record.material_askon.id if record.material_askon else None,
            'material_askon__field0': record.material_askon.field0 if record.material_askon else '',
            'material_id': record.material.id if record.material else None,
            'material__name': record.material.name if record.material else '',
            'complex_name': record.complex_name,
            'complex_gost': record.complex_gost,
            'edizm_id': record.edizm.id,
            'edizm__code': record.edizm.code,
            'edizm__name': record.edizm.name,
            'qty': record.qty,
            'props': record.props._value,
            'enabled': IsBitOn(record.props, 0),
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Launch_operations_materialQuerySet(self.model, using=self._db)


class Launch_operations_material(AuditModel):
    operation_material = ForeignKeySetNull(Operation_material, null=True, blank=True)
    launch_operationitem = ForeignKeyCascade(Launch_operations_item)
    material = ForeignKeyProtect(Materials, null=True, blank=True)
    material_askon = ForeignKeyProtect(Material_askon, null=True, blank=True)
    edizm = ForeignKeyProtect(Ed_izm)
    qty = DecimalField(max_digits=10, decimal_places=4, default=0.0)

    props = BitField(flags=(
        ('enabled', 'Доступность в данной производственной спецификации'),  # 1
    ), default=1, db_index=True)

    @property
    def complex_name(self):
        if self.material:
            return f'{self.material.materials_type.full_name}{self.material.full_name}'
        else:
            return ''

    @property
    def complex_gost(self):
        if self.material:
            if self.material.materials_type.gost:
                return f'{self.material.materials_type.gost} / {self.material.gost}'
            else:
                return self.material.gost if self.material else ''
        else:
            return ''

    objects = Launch_operations_materialManager()

    def __str__(self):
        return f"{self.complex_name}"

    class Meta:
        verbose_name = 'Кросс таблица'



import logging

from django.core.management import BaseCommand
from isc_common.logger.Logger import Logger

from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = 'Создание товаоных позиций после импорта из Лоцмана'
    def handle(self, *args, **options):
        try:
            from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcyManager
            Lotsman_documents_hierarcyManager.refresh_lotsman_kd()
        except Exception as ex:
            raise ex



import logging

from django.db import ProgrammingError

logger = logging.getLogger(__name__)


class Operation_typesStack:
    def get_first_item_of_tuple(self, tp):
        res, _ = tp
        return res

    def __init__(self):
        from kaf_pas.planing.models.operation_types import Operation_types

        try:
            self.ASSEMBLY_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='AS_TSK', defaults=dict(
                props=Operation_types.props.plus,
                name='Задание на комплектацию',
                editing=False,
                deliting=False,
            )))

            self.PRODUCTION_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='PRD_TSK', defaults=dict(
                props=Operation_types.props.plus,
                name='Задание на производство',
                editing=False,
                deliting=False,
            )))

            self.POSTING_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='PST_TSK', defaults=dict(
                props=0,
                name='Оприходование',
                editing=True,
                deliting=True,
            )))

            self.POSTING_DETAIL_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='DETAIL_PST_TSK', defaults=dict(
                props=Operation_types.props.plus,
                name='Детализация Оприходывания',
                editing=True,
                deliting=True,
            )))

            self.WRITE_OFF_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='WRT_OFF_TSK', defaults=dict(
                props=0,
                name='Списание',
                editing=True,
                deliting=True,
            )))

            self.WRITE_DETAIL_OFF_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='DETAIL_WRT_OFF_TSK', defaults=dict(
                props=Operation_types.props.minus,
                name='Детализация Списания',
                editing=True,
                deliting=True,
            )))

            self.ROUTING_TASK = self.get_first_item_of_tuple(Operation_types.objects.update_or_create(code='RT_TSK', defaults=dict(
                props=0,
                name='Маршрутизация',
                editing=False,
                deliting=False,
            )))
        except ProgrammingError as ex:
            logger.warning(ex)



import logging

from django.conf import settings
from django.db import transaction, connection
from django.db.models import TextField
from django.forms import model_to_dict

from isc_common import StackElementNotExist
from isc_common.common.mat_views import create_tmp_mat_view, drop_mat_view
from isc_common.datetime import DateToStr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from isc_common.progress import Progress
from isc_common.ws.progressStack import ProgressStack
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.status_operation_types import Status_operation_types
from kaf_pas.production.models.launch_operation_material import Launch_operations_material
from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
from kaf_pas.production.models.launch_operations_item import Launch_operations_item

logger = logging.getLogger(__name__)


class OperationsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Rout_item():

    def __init__(self, item_id, first_operation, last_operation):
        self.item_id = item_id
        self.first_operation = first_operation
        self.last_operation = last_operation


class OperationsManager(AuditManager):

    @staticmethod
    def delete_recursive(operation, soft_delete=False, user=None):
        with transaction.atomic():
            from kaf_pas.planing.models.operation_refs import Operation_refs

            progress = Progress(
                id=operation.id,
                qty=Operation_refs.objects.get_descendants_count(id=operation.id),
                user=user,
                message='Удаление операций',
                title='Выполнено'
            )
            try:

                for item in Operation_refs.objects.get_descendants(id=operation.id, order_by_clause='order by level desc'):
                    if not soft_delete:
                        Operations.objects.filter(id=item.child_id).delete()
                    else:
                        Operations.objects.filter(id=item.child_id).soft_delete()

                    if progress:
                        progress.step()

                if progress:
                    progress.close()
            except Exception as ex:
                if progress:
                    progress.close()
                raise ex

    def deleteFromRequest(self, request, removed=None, ):

        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    for operation in super().filter(id=id):
                        OperationsManager.delete_recursive(operation=operation)
                        operation.delete()
                        res += 1
        return res

    @staticmethod
    def make_routing(data):
        from isc_common.auth.models.user import User
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.planing.models.operation_level import Operation_level
        from kaf_pas.planing.models.operation_locations import Operation_locations
        from kaf_pas.planing.models.operation_material import Operation_material
        from kaf_pas.planing.models.operation_refs import Operation_refs
        from kaf_pas.planing.models.operation_resources import Operation_resources
        from kaf_pas.planing.models.operation_standard_prod import Operation_standard_prod
        from kaf_pas.planing.models.operation_value import Operation_value
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.planing.models.operation_item import Operation_item
        from kaf_pas.ckk.models.ed_izm import Ed_izm

        logger.debug(f'data: {data}')

        OperationsManager.clean_routing(data=data)

        launch_id = data.get('id')

        launch = Launches.objects.get(id=launch_id)

        user = User.objects.get(id=data.get('user_id'))
        edizm = Ed_izm.objects.get(code='шт')

        progress = ProgressStack(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
        )

        id_progress1 = f'launch_{launch.id}_{user.id}'
        try:

            demand_str = f'<h3>Расчет маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3>'

            sql_str = f'''with recursive r as (
                                                    select *,
                                                           1 as level
                                                    from production_launch_item_refs
                                                    where parent_id is null
                                                      and launch_id = {launch_id}
                                                    union all
                                                    select production_launch_item_refs.*,
                                                           r.level + 1 as level
                                                    from production_launch_item_refs
                                                             join r
                                                                  on
                                                                      production_launch_item_refs.parent_id = r.child_id)
                                                
                                                select r1.id,
                                                       r1.parent_id,
                                                       r1.child_id,
                                                       r1.launch_id,
                                                       r1.stmp1,
                                                       r1.stmp2,
                                                       r1.qty,
                                                       r1.level
                                                from (select distinct r.id,
                                                                      r.parent_id,
                                                                      r.child_id,
                                                                      stmp1.value_str                         stmp1,
                                                                      stmp2.value_str                         stmp2,
                                                                      r.launch_id,
                                                                      (
                                                                          select coalesce(kda.value_int, kda.value_str::numeric(19, 4), 0)
                                                                          from production_launch_item_line plil
                                                                                   join kd_document_attributes kda on kda.id = plil."SPC_CLM_COUNT_id"
                                                                          where plil.child_id = r.child_id
                                                                            and plil.parent_id = r.parent_id
                                                                            and plil.launch_id = r.launch_id) qty,
                                                                      (
                                                                          select plil.section
                                                                          from production_launch_item_line plil
                                                                          where plil.child_id = r.child_id
                                                                            and plil.parent_id = r.parent_id
                                                                            and plil.launch_id = r.launch_id) section,
                                                                      level
                                                      from r
                                                               join ckk_item ci on ci.id = r.child_id
                                                               join kd_document_attributes stmp1 on stmp1.id = ci."STMP_1_id"
                                                               join kd_document_attributes stmp2 on stmp2.id = ci."STMP_2_id"
                                                      where r.launch_id = {launch_id}
                                                      order by level desc) r1
                                                where lower(r1.section) != 'документация'
    
                                                        '''
            with transaction.atomic():

                mat_view_name = create_tmp_mat_view(sql_str=sql_str, indexes=['parent_id', 'child_id'])
                with connection.cursor() as cursor:

                    cursor.execute(f'select count(*) from {mat_view_name}')
                    count, = cursor.fetchone()

                    logger.debug(f'{mat_view_name} count(*): {count}')

                    progress.show(
                        title=f'<b>Обработано товарных позиций</b>',
                        label_contents=demand_str,
                        cntAll=count * 2,
                        id=id_progress1
                    )

                    cursor.execute(f'select * from {mat_view_name} order by level desc')
                    rows = cursor.fetchall()
                    cnt = 0
                    from isc_common import Stack
                    outcome_items = Stack()

                    for row in rows:
                        def make_oparetions(row, mode='chlid'):
                            id, parent_id, child_id, launch_id, stmp1, stmp2, qty, level = row

                            if mode == 'child':
                                cursor.execute(f'select sum(qty) from {mat_view_name} where child_id = %s', [child_id])
                                qty, = cursor.fetchone()
                            elif mode == 'parent':
                                child_id = parent_id

                            if len(outcome_items.find(lambda outcome_item: outcome_item.item_id == child_id)) == 0:
                                income_operation = None
                                cnt1 = Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).count()
                                if cnt1 > 0:
                                    first_operation = None
                                    for launch_operations_item in Launch_operations_item.objects.filter(item_id=child_id, launch_id=launch_id).order_by('num'):
                                        outcome_operation = Operations.objects.create(
                                            opertype=settings.OPERS_TYPES_STACK.ROUTING_TASK,
                                        )

                                        Operation_launches.objects.create(operation=outcome_operation, launch=launch)
                                        Operation_item.objects.create(operation=outcome_operation, item=launch_operations_item.item)
                                        Operation_level.objects.create(operation=outcome_operation, level=level)
                                        if qty != None:
                                            Operation_value.objects.create(operation=outcome_operation, value=qty, edizm=edizm)

                                        if income_operation == None:
                                            first_operation = outcome_operation

                                        for launch_operation_material in Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item):
                                            if launch_operation_material.material_askon != None:
                                                Operation_material.objects.get_or_create(operation=outcome_operation, material=launch_operation_material.material_askon)
                                            if launch_operation_material.material != None:
                                                Operation_standard_prod.objects.get_or_create(operation=outcome_operation, material=launch_operation_material.material)

                                        if Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item) == 0:
                                            raise Exception(f'Для : {stmp1} : {stmp2} ID={child_id} не задан ресурс.')

                                        for launch_operation_resources in Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item):
                                            if launch_operation_resources.resource != None:
                                                Operation_resources.objects.get_or_create(operation=outcome_operation, resource=launch_operation_resources.resource)
                                            if launch_operation_resources.location != None:
                                                Operation_locations.objects.get_or_create(operation=outcome_operation, location=launch_operation_resources.location)

                                        Operation_refs.objects.get_or_create(parent=income_operation, outcome=outcome_operation)
                                        income_operation = outcome_operation
                                        cnt1 -= 1
                                        if cnt1 == 0:
                                            outcome_items.push(Rout_item(item_id=child_id, first_operation=first_operation, last_operation=outcome_operation))
                                else:
                                    raise Exception(f'Для : {stmp1} : {stmp2} ID={child_id} не задано ни одной операции. Запустите анализатор готовности к запуску.')

                        make_oparetions(row=row)

                        cnt += 1
                        progress.setCntDone(cnt=cnt, id=id_progress1)

                    for row in rows:
                        id, parent_id, child_id, launch_id, stmp1, stmp2, qty, level = row

                        try:
                            first_operation = outcome_items.find_one(lambda outcome_item: outcome_item.item_id == parent_id)
                        except StackElementNotExist:
                            make_oparetions(row=row, mode='parent')
                            first_operation = outcome_items.find_one(lambda outcome_item: outcome_item.item_id == parent_id)

                        cursor.execute(f'''select child_id, stmp1, stmp2 from {mat_view_name} where parent_id = %s''', [parent_id])
                        parents_rows = cursor.fetchall()
                        for parents_row in parents_rows:
                            _child_id, stmp1, stmp2 = parents_row
                            _child = outcome_items.find_one(lambda item: item.item_id == _child_id)

                            Operation_refs.objects.get_or_create(parent=_child.last_operation, outcome=first_operation.last_operation)
                            deleted, _ = Operation_refs.objects.filter(parent__isnull=True, outcome=first_operation.last_operation).delete()
                        progress.setCntDone(cnt=cnt, id=id_progress1)
                        cnt += 1

                    progress.close(id=id_progress1)
                    drop_mat_view(mat_view_name)

                launch.props |= Launches.props.routing_made
                launch.save()
                settings.EVENT_STACK.EVENTS_PRODUCTION_MAKE_ROUTING.send_message(f'<h3>Выполнен Расчет маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3><p/>')

        except Exception as ex:
            progress.close(id=id_progress1)
            raise ex

    def clean_routing(data):
        from kaf_pas.production.models.launches import Launches
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from isc_common.auth.models.user import User

        logger.debug(f'data: {data}')

        launch_id = data.get('id')
        launch = Launches.objects.get(id=launch_id)

        cntAll = Operation_launches.objects.filter(launch=launch).count()

        if cntAll > 0:
            user = User.objects.get(id=data.get('user_id'))

            progress = ProgressStack(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=f'common_{user.username}',
            )

            id_progress1 = f'launch_{launch.id}_{user.id}'
            try:
                with transaction.atomic():
                    demand_str = f'<h3>Удаление маршрутизации : Запуск № {launch.code} от {DateToStr(launch.date)}</h3>'

                    launch.props &= ~Launches.props.routing_made
                    launch.save()

                    progress.show(
                        title=f'<b>Обработано товарных позиций</b>',
                        label_contents=demand_str,
                        cntAll=cntAll,
                        id=id_progress1
                    )

                    cnt = 0

                    for operation_launches in Operation_launches.objects.filter(launch=launch):
                        operation_launches.operation.delete()
                        progress.setCntDone(cnt=cnt, id=id_progress1)
                        cnt += 1

                    progress.close(id=id_progress1)
                    settings.EVENT_STACK.EVENTS_PRODUCTION_DELETE_ROUTING.send_message(f'<h3>Выполнено Удаление маршрутизации: Запуск № {launch.code} от {DateToStr(launch.date)}</h3><p/>')
            except Exception as ex:
                progress.close(id=id_progress1)
                raise ex

        return model_to_dict(launch)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'opertype_id': record.opertype.id,
            'opertype__full_name': record.opertype.full_name,
            'status_id': record.status.id if record.status else None,
            'status__code': record.status.code if record.status else None,
            'status__name': record.status.name if record.status else None,
        }
        return res

    def get_queryset(self):
        return OperationsQuerySet(self.model, using=self._db)


class Operations(AuditModel):
    opertype = ForeignKeyProtect(Operation_types)
    status = ForeignKeyProtect(Status_operation_types, null=True, blank=True)
    description = TextField(null=True, blank=True)

    objects = OperationsManager()

    def __str__(self):
        return f"ID:{self.id}, opertype: [{self.opertype}], status: [{self.status}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Опреации системные'



# Generated by Django 2.2.8 on 2019-12-14 15:40

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0104_auto_20191214_1537'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='kd.Lotsman_documents_hierarcy'),
        ),
    ]



import datetime
import logging

from django.conf import settings
from django.db import transaction
from django.db.models import DateTimeField, PositiveIntegerField, F, BooleanField

from isc_common import getAttr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from isc_common.number import DelProps
from isc_common.ws.progressStack import ProgressStack
from kaf_pas.production.models.launches import LaunchesManager
from kaf_pas.production.models.status_launch import Status_launch
from kaf_pas.sales.models.demand import Demand
from kaf_pas.sales.models.demand_view import Demand_view

logger = logging.getLogger(__name__)


class Launches_viewQuerySet(BaseRefQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Launches_viewManager(BaseRefManager):
    @staticmethod
    def make_launch(data):
        from isc_common.auth.models.user import User
        from kaf_pas.ckk.models.item_refs import Item_refs

        demand_id = getAttr(data, 'demand_id')
        qty = getAttr(data, 'qty')
        code = getAttr(data, 'code')
        date = getAttr(data, 'date')
        date = datetime.datetime.strptime(date, '%Y-%m-%d')
        user = User.objects.get(id=getAttr(data, 'user_id'))
        status, _ = Status_launch.objects.get_or_create(name='Формирование')

        with transaction.atomic():
            for demand in Demand.objects.select_for_update().filter(id=demand_id):
                tail_qty = Demand_view.objects.get(id=demand_id).tail_qty
                if tail_qty < qty:
                    raise Exception(f'Затребованное количестово для запуска ({qty}), превышает возможное к запуску ({tail_qty})')

                progress = ProgressStack(
                    host=settings.WS_HOST,
                    port=settings.WS_PORT,
                    channel=f'common_{user.username}',
                )

                id = f'demand_{demand.id}'
                demand_str = f'<h3>Формирование запуска: Заказ № {demand.code} от {demand.date}</h3>'

                progress.show(
                    title=f'<b>Обработано товарных позиций</b>',
                    label_contents=demand_str,
                    cntAll=Item_refs.objects.get_descendants_count(id=demand.precent_item.item.id),
                    id=id
                )

                cnt = 0
                Item_refs.objects.filter(child=F('parent')).delete()
                launch = Launches_view.objects.create(
                    code=code,
                    demand_id=demand_id,
                    qty=qty,
                    date=date,
                    status=status
                )
                for item_ref in Item_refs.objects.get_descendants(id=demand.precent_item.item.id):
                    print(item_ref)

        return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        return Launches_viewManager.make_launch(request.get_data())

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'date': record.date,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,

            'demand_id': record.demand.id,
            'demand__code': record.demand.code,
            'demand__date': record.demand.date,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'qty': record.qty,
            'routing_made': record.props.routing_made,
            'isFolder': record.isFolder,
            'props': record.props,

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    def get_queryset(self):
        return Launches_viewQuerySet(self.model, using=self._db)


class Launches_view(BaseRefHierarcy):
    date = DateTimeField()
    status = ForeignKeyProtect(Status_launch)
    demand = ForeignKeyProtect(Demand)
    qty = PositiveIntegerField()
    isFolder = BooleanField()
    props = LaunchesManager.props()

    objects = Launches_viewManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Запуски'
        db_table = 'production_launches_view'
        managed = False



# Generated by Django 2.2.8 on 2019-12-08 07:08

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0046_auto_20191207_2357'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_refs',
            name='child',
        ),
        migrations.RemoveField(
            model_name='operation_refs',
            name='parent',
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='income',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, related_name='operation_income', to='planing.Operations'),
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='outcome',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='operation_outcome', to='planing.Operations'),
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcyManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        try:
            Lotsman_documents_hierarcyManager.make_mview()


        except Exception as ex:
            raise ex



# Generated by Django 2.2.8 on 2019-12-13 23:51

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0099_auto_20191213_2032'),
    ]

    operations = [
        migrations.CreateModel(
            name='Lotsman_documents_hierarcy',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Documents')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents_hierarcy')),
            ],
            options={
                'verbose_name': 'Иерархия документа из Лоцмана',
            },
        ),
        migrations.CreateModel(
            name='Lotsman_document_attr_cross',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position_in_document', models.PositiveIntegerField()),
                ('section', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Раздел')),
                ('subsection', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Подраздел')),
                ('attribute', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Document_attributes')),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Lotsman_documents_hierarcy')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
                'unique_together': {('document', 'position_in_document')},
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-28 20:56

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0061_auto_20190828_1719'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_resources',
            name='location',
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
    ]



import logging

from bitfield import BitField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager

from kaf_pas.ckk.models.item import Item

logger = logging.getLogger(__name__)


class Item_refsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def _check(self, **kwargs):
        if kwargs.get('parent') != None:
            if isinstance(kwargs.get('parent'), Item):
                parent = kwargs.get('parent').id
            else:
                parent = kwargs.get('parent')

            if isinstance(kwargs.get('child'), Item):
                child = kwargs.get('child').id
            else:
                child = kwargs.get('child')

            if parent == child:
                raise Exception(f'Attempt to write circular reference id ({parent})')

    def create(self, **kwargs):
        self._check(**kwargs)
        return super().create(**kwargs)

    def update(self, **kwargs):
        self._check(**kwargs)
        return super().update(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().get_or_create(defaults, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().update_or_create(defaults, **kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_refsManager(AuditManager):

    @staticmethod
    def props():
        return BitField(flags=(
            ('relevant', 'Актуальность'),  # 1
        ), default=3, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_refsQuerySet(self.model, using=self._db)


class Item_refs(AuditModel):
    child = ForeignKeyProtect(Item, related_name='child')
    parent = ForeignKeyProtect(Item, related_name='parent', blank=True, null=True)

    props = Item_refsManager.props()

    objects = TreeAuditModelManager()

    def __str__(self):
        return f'\nID={self.id}, child=[{self.child}], parent=[{self.parent}]'

    class Meta:
        verbose_name = 'Item_refs'
        unique_together = ('child', 'parent')



import logging

from django.core.management import BaseCommand
from django.db import transaction, connection

from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refs
from tqdm import tqdm

from kaf_pas.ckk.models.item_refs_hierarcy import Item_refs_hierarcy

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Заполнение поля section_num"

    def dublicate_childs(self, parent_id, syntetic_parent_id):
        for item_refs in Item_refs.objects.filter(parent_id=parent_id):
            res = Item_refs_hierarcy.objects.create(
                id=item_refs.child.id,
                parent_id=syntetic_parent_id,
                item_refs=item_refs
            )
            logger.debug(res)

            # if Item_refs.objects.filter(parent=item_refs.child).count() > 0:
            #     raise Exception(f'item_refs.child have parents')

    def handle(self, *args, **options):
        logger.info(self.help)
        with transaction.atomic():
            with connection.cursor() as cursor:
                Item_refs_hierarcy.objects.all().delete()
                cursor.execute('''select child_id,
                                         count(*)
                                   from ckk_item_refs 
                                    group by child_id
                                    having count(*) > 1''')
                rows_dbls = cursor.fetchall()

                cursor.execute('''select child_id,
                                           count(*)
                                   from ckk_item_refs 
                                    group by child_id
                                    having count(*) = 1''')
                rows_ones = cursor.fetchall()

                self.pbar = tqdm(total=len(rows_dbls) + len(rows_ones))

                print('row_dbls')

                for row_dbls in rows_dbls:
                    child_id, _ = row_dbls

                    cursor.execute('''select id, parent_id
                                       from ckk_item_refs 
                                        where child_id = %s''', [child_id])

                    child_rows = cursor.fetchall()
                    step = 1
                    for child_row in child_rows:
                        item_refs_id, parent_id = child_row

                        if step == 1:
                            res = Item_refs_hierarcy.objects.create(
                                id=child_id,
                                parent_id=parent_id,
                                item_refs_id=item_refs_id
                            )
                            logger.debug(res)
                            step += 1
                        else:
                            cursor.execute(f"select nextval('ckk_item_id_seq')")
                            syntetic_child_id, = cursor.fetchone()

                            res = Item_refs_hierarcy.objects.create(
                                id=syntetic_child_id,
                                parent_id=parent_id,
                                item_refs_id=item_refs_id
                            )
                            logger.debug(res)

                            # self.dublicate_childs(parent_id=child_id, syntetic_parent_id=syntetic_child_id)

                    if self.pbar:
                        self.pbar.update(1)

                print('rows_ones')
                for row_ones in rows_ones:
                    child_id, _ = row_ones

                    item_refs = Item_refs.objects.get(child_id=child_id)
                    res = Item_refs_hierarcy.objects.create(
                        id=child_id,
                        parent_id=item_refs.parent.id if item_refs.parent else None,
                        item_refs=item_refs
                    )
                    logger.debug(res)

                    if self.pbar:
                        self.pbar.update(1)


                if self.pbar:
                    self.pbar.close()






# Generated by Django 2.2.1 on 2019-05-24 09:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0030_remove_demand_item'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='demand',
            name='customer',
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-18 14:00

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0106_auto_20191118_1350'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_line',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'),), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'),), db_index=True, default=1),
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from kaf_pas.ckk.models.item_line import Item_line, Item_lineManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Заполнение поля section_num"

    def handle(self, *args, **options):
        logger.info(self.help)

        self.pbar = tqdm(total=Item_line.objects.all().count())

        with transaction.atomic():
            for item in Item_line.objects.all():
                item.section_num = Item_lineManager.section_num(item.section)
                item.save()

                if self.pbar:
                    self.pbar.update(1)

        if self.pbar:
            self.pbar.close()



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.planing.models.status_operation_types import Status_operation_types, Status_operation_typesManager


@JsonResponseWithException()
def Status_operation_types_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Status_operation_types.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Status_operation_typesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Add(request):
    return JsonResponse(DSResponseAdd(data=Status_operation_types.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Update(request):
    return JsonResponse(DSResponseUpdate(data=Status_operation_types.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Status_operation_types.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Status_operation_types.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Info(request):
    return JsonResponse(DSResponse(request=request, data=Status_operation_types.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Status_operation_types_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Status_operation_types.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            



import logging
import os

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from isc_common import setAttr
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_image_refs import Item_image_refs
from kaf_pas.ckk.models.item_line import Item_line
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.kd.models.cdw_attrs import Cdw_attrs
from kaf_pas.kd.models.cdws import Cdws
from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_thumb import Documents_thumb
from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
from kaf_pas.kd.models.pdfs import Pdfs
from kaf_pas.kd.models.spw_attrs import Spw_attrs, Spw_attrsQuerySet
from kaf_pas.kd.models.spws import Spws

logger = logging.getLogger(__name__)
logger1 = logging.getLogger(f'{__name__}_1')


class MakeItem:
    def meke_cdw(self, command_style=True):
        if command_style:
            self.pbar = tqdm(total=Cdws.objects.filter(props=~Documents.props.beenItemed).count())

        # <editor-fold desc="Этап 1">
        with transaction.atomic():
            for document in Cdws.objects.filter(props=~Documents.props.beenItemed):
                if document.file_document.lower().find('мусор') == -1:
                    # logger.debug(f'Iteming: {document}')

                    STMP_1 = Cdw_attrs.objects.all().get_attr(document=document, code='STMP_1')
                    STMP_2 = Cdw_attrs.objects.all().get_attr(document=document, code='STMP_2')

                    if STMP_1 != None or STMP_2 != None:

                        args = dict(
                            STMP_1_id=STMP_1.id if STMP_1 else None,
                            STMP_2_id=STMP_2.id if STMP_2 else None,
                            props=Item.props.relevant
                        )

                        cnt = 0
                        items = []

                        for item in Item.objects.filter(**args):

                            dir, file_name = os.path.split(document.file_document)
                            dir1, file_name1 = os.path.split(item.document.file_document)
                            if file_name == file_name1 and document.file_size == item.document.file_size:
                                items.append(item)
                                cnt += 1

                        if cnt == 0:
                            item, created = Item.objects.get_or_create(**args, defaults=dict(document=document, props=Item.props.relevant | Item.props.from_cdw))

                            if created:
                                logger.debug(f'\nAdded: {item}, cdw')
                            else:
                                logger.debug('==============================================================================================================')

                            try:
                                Item_refs.objects.get_or_create(child=item)
                            except Item_refs.MultipleObjectsReturned:
                                ...

                            self.link_image_to_item(item)

                            # item.delete_soft()
                        else:
                            for item in items:
                                logger.debug(f'\nНайдено: элемент пришедший из {cnt + 1}-x источников : {item}')
                                self.link_image_to_item(item)

                    else:
                        logger.debug(f'\nSTMP_1 is None and STMP_2 is None, document: {document}')
                        document.delete_soft()

                document.props |= Documents.props.beenItemed
                document.save()

                if command_style and self.pbar:
                    self.pbar.update(1)

        if command_style and self.pbar:
            self.pbar.close()


class Command(BaseCommand):
    help = "Загрузка составов изделий"
    makeItem = MakeItem()

    def link_image_to_item(self, item):
        for document_thumb in Documents_thumb.objects.filter(document=item.document):
            item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb=document_thumb)
            # logger.debug(f'\nItem_image_refs: {item_image_refs}, created: {created}')

        for document_thumb10 in Documents_thumb10.objects.filter(document=item.document):
            item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb10=document_thumb10)
            # logger.debug(f'\nItem_image_refs: {item_image_refs}, created: {created}')

    def rec_image_cwd(self, item_id, STMP_2):
        for item_image in Item_image_refs.objects.select_related('item').filter(item__props__in=[Item.props.relevant | Item.props.from_cdw, Item.props.relevant | Item.props.from_pdf], item__STMP_2__value_str=STMP_2):
            _, created = Item_image_refs.objects.get_or_create(item_id=item_id, thumb_id=item_image.thumb_id, thumb10_id=item_image.thumb10_id)
            # if created:
            #     logger1.debug(f'\nAdded: {item_image}')

    def del_blancks1(self, STMP_2):
        STMP_2_1 = ' '.join(STMP_2.split())
        if STMP_2_1 != STMP_2:
            return STMP_2_1
        return STMP_2

    def handle(self, *args, **options):

        logger.info(self.help)

        self.makeItem.meke_cdw(True)

        return
        # </editor-fold>

        # <editor-fold desc="Этап 2">
        self.pbar = tqdm(total=Pdfs.objects.filter(props=~Documents.props.beenItemed).count())

        STMP_1_type = Attr_type.objects.get(code='STMP_1')
        STMP_2_type = Attr_type.objects.get(code='STMP_2')

        with transaction.atomic():
            for document in Pdfs.objects.filter(props=~Documents.props.beenItemed):
                if document.file_document.lower().find('мусор') == -1:
                    # logger.debug(f'Iteming: {document}')

                    full_path = document.file_document
                    _, file_name = os.path.split(full_path)
                    file_name_part = file_name.split(' - ')
                    if len(file_name_part) == 2:

                        STMP_1, ext = os.path.splitext(file_name_part[1].strip())
                        STMP_2 = self.del_blancks1(file_name_part[0].strip())
                    else:
                        STMP_1 = str(file_name.strip()).replace('.pdf', '')
                        STMP_2 = STMP_1

                    args = dict(
                        STMP_1=STMP_1,
                        STMP_2=STMP_2,
                    )

                    cnt = 0
                    items = []
                    for item in Item.objects.filter(
                            STMP_1__value_str=args.get('STMP_1'),
                            STMP_2__value_str=args.get('STMP_2'),
                            props=Item.props.relevant
                    ):

                        dir, file_name = os.path.split(document.file_document)
                        dir1, file_name1 = os.path.split(item.document.file_document)
                        if file_name == file_name1:
                            items.append(item)
                            cnt += 1

                    if cnt == 0:

                        STMP_1_attr, _ = Document_attributes.objects.get_or_create(value_str=STMP_1, attr_type=STMP_1_type)
                        STMP_2_attr, _ = Document_attributes.objects.get_or_create(value_str=STMP_2, attr_type=STMP_2_type)

                        item, created = Item.objects.get_or_create(
                            STMP_1=STMP_1_attr,
                            STMP_2=STMP_2_attr,
                            props=Item.props.relevant,
                            defaults=dict(document=document, props=Item.props.relevant | Item.props.from_pdf)
                        )

                        if created:
                            logger.debug(f'\nAdded: {item}')
                        else:
                            logger.debug('==============================================================================================================')

                        self.link_image_to_item(item)
                    else:
                        for item in items:
                            logger.debug(f'\nНайдено: элемент пришедший из {cnt + 1}-x источников : {item}')
                            self.link_image_to_item(item)

                    document.props |= Documents.props.beenItemed
                    document.save()

                if self.pbar:
                    self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

        return
        # </editor-fold>

        self.pbar = tqdm(total=Spws.objects.filter(props=~Documents.props.beenItemed).count())

        # <editor-fold desc="Этап 3">
        with transaction.atomic():
            top_auto_level_type, _ = Attr_type.objects.get_or_create(code='top_auto_level')
            top_auto_level_attr, _ = Document_attributes.objects.get_or_create(attr_type=top_auto_level_type, value_str='Автоматически сгененрированный состав изделий')
            top_auto_level_item, _ = Item.objects.get_or_create(STMP_1=top_auto_level_attr, props=Item.props.relevant | Item.props.from_spw)
            item_refs, _ = Item_refs.objects.get_or_create(child=top_auto_level_item)

            for document in Spws.objects.filter(props=~Documents.props.beenItemed):
                # with transaction.atomic():
                if document.file_document.lower().find('мусор') == -1:
                    # logger.debug(f'Iteming: {document}')

                    STMP_1 = Spw_attrs.objects.all().get_attr(document=document, code='STMP_1')
                    STMP_2 = Spw_attrs.objects.all().get_attr(document=document, code='STMP_2')

                    if STMP_1 != None or STMP_2 != None:

                        kwargs = dict(
                            STMP_1_id=STMP_1.id if STMP_1 else None,
                            STMP_2_id=STMP_2.id if STMP_2 else None,
                            props=Item.props.relevant
                        )

                        parent, created = Item.objects.get_or_create(
                            **kwargs,
                            defaults=dict(document=document, props=Item.props.relevant | Item.props.from_spw))

                        if created:
                            item_refs = Item_refs.objects.create(parent=top_auto_level_item, child=parent)
                            logger.debug(f'\nAdded: {item_refs} as tpo level')

                        if kwargs.get('STMP_2__value_str') != None:
                            self.rec_image_cwd(parent.id, kwargs.get('STMP_2__value_str'))
                        self.link_image_to_item(parent)

                        query_spw = Spw_attrs.objects.filter(document=document, props=Document_attributes.relevant).order_by(*['position_in_document'])
                        specification = Spw_attrsQuerySet.make_specification(queryResult=query_spw)

                        SPC_CLM_MARK_old = None
                        for line_specification in specification:
                            SPC_CLM_NAME = line_specification.get('SPC_CLM_NAME')
                            SPC_CLM_NAME_ID = line_specification.get('SPC_CLM_NAME_ID')

                            SPC_CLM_MARK = line_specification.get('SPC_CLM_MARK')
                            SPC_CLM_MARK_ID = line_specification.get('SPC_CLM_MARK_ID')

                            if SPC_CLM_NAME_ID != None or SPC_CLM_MARK_ID != None:

                                if isinstance(SPC_CLM_MARK, str) and SPC_CLM_MARK.startswith('-') and isinstance(SPC_CLM_MARK_ID, int):
                                    SPC_CLM_MARK = f'{SPC_CLM_MARK_old}{SPC_CLM_MARK}'
                                    for document_attribute in Document_attributes.objects.filter(id=SPC_CLM_MARK_ID):
                                        try:
                                            _document_attribute = Document_attributes.objects.get(attr_type=document_attribute.attr_type, value_str=SPC_CLM_MARK, props=document_attribute.props)
                                            Document_attr_cross.objects.filter(attribute=document_attribute).update(attribute=_document_attribute)
                                            setAttr(line_specification, 'SPC_CLM_MARK_ID', _document_attribute.id)
                                            document_attribute.delete()
                                        except Document_attributes.DoesNotExist:
                                            document_attribute.value_str = SPC_CLM_MARK
                                            document_attribute.save()
                                            logger1.debug(f'=================================================>>>>>> Uppdated: {SPC_CLM_MARK}')

                                elif isinstance(SPC_CLM_MARK, str):
                                    SPC_CLM_MARK_old = SPC_CLM_MARK

                                child_args = dict(
                                    STMP_1=SPC_CLM_NAME_ID,
                                    STMP_2=SPC_CLM_MARK_ID,
                                )

                                child, created = Item.objects.get_or_create(
                                    STMP_1_id=child_args.get('STMP_1'),
                                    STMP_2_id=child_args.get('STMP_2'),
                                    props__in=[Item.props.relevant, Item.props.relevant | Item.props.from_spw | Item.props.for_line],
                                    defaults=dict(document=document)
                                )

                                if created:
                                    if child.STMP_2 != None:
                                        self.rec_image_cwd(child.id, child.STMP_2.value_str)
                                    logger.debug(f'\nAdded: {child} as child')
                                else:
                                    logger.debug(f'\nNot Added: {child} as child')

                                item_refs, created = Item_refs.objects.get_or_create(parent=parent, child=child)
                                if created:
                                    logger.debug(f'\nAdded: {item_refs}')
                                else:
                                    logger.debug(f'\nNot Added: {item_refs}')

                                defaults = dict(
                                    parent=parent,
                                    child=child,
                                    SPC_CLM_FORMAT_id=line_specification.get('SPC_CLM_FORMAT_ID'),
                                    SPC_CLM_ZONE_id=line_specification.get('SPC_CLM_ZONE_ID'),
                                    SPC_CLM_POS_id=line_specification.get('SPC_CLM_POS_ID'),
                                    SPC_CLM_MARK_id=line_specification.get('SPC_CLM_MARK_ID'),
                                    SPC_CLM_NAME_id=line_specification.get('SPC_CLM_NAME_ID'),
                                    SPC_CLM_COUNT_id=line_specification.get('SPC_CLM_COUNT_ID'),
                                    SPC_CLM_NOTE_id=line_specification.get('SPC_CLM_NOTE_ID'),
                                    SPC_CLM_MASSA_id=line_specification.get('SPC_CLM_MASSA_ID'),
                                    SPC_CLM_MATERIAL_id=line_specification.get('SPC_CLM_MATERIAL_ID'),
                                    SPC_CLM_USER_id=line_specification.get('SPC_CLM_USER_ID'),
                                    SPC_CLM_KOD_id=line_specification.get('SPC_CLM_KOD_ID'),
                                    SPC_CLM_FACTORY_id=line_specification.get('SPC_CLM_FACTORY_ID'),

                                    section=line_specification.get('section'),
                                    subsection=line_specification.get('subsection'),
                                )

                                item_line, created = Item_line.objects.get_or_create(parent=parent, child=child, defaults=defaults)
                                if created:
                                    logger.debug(f'\nAdded: {item_line} as item_line')
                                else:
                                    logger.debug(f'\nNot Added: {item_line} as item_line')

                    else:
                        logger.debug(f'\nSTMP_1 and STMP_2 is None, document: {document}')
                        document.delete_soft()

                document.props |= Documents.props.beenItemed
                document.save()

                if self.pbar:
                    self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

        logger.info("Загрузка выполнена.")
        # </editor-fold>

# with transaction.atomic():



import logging

from events.events_manager import EventsManager

logger = logging.getLogger(__name__)


class EventStack:
    eventsManager = EventsManager()

    def __init__(self):
        self.EVENTS_TEST = self.eventsManager.get_event('events.test', 'События.Test', compulsory_reading=True)

        self.EVENTS_UPLOAD_PDF_NEW = self.eventsManager.get_event('events.upload.pdf.new', 'События.Закачка.PDF.Новый', compulsory_reading=True)
        self.EVENTS_UPLOAD_PDF_MODIFIED = self.eventsManager.get_event('events.upload.pdf.modified', 'События.Закачка.PDF.Исправленный', compulsory_reading=True)
        #
        self.EVENTS_UPLOAD_KOMPAS_NEW = self.eventsManager.get_event('events.upload.kompas.new', 'События.Закачка.КОМПАС.Новый', compulsory_reading=True)
        self.EVENTS_UPLOAD_KOMPAS_MODIFIED = self.eventsManager.get_event('events.upload.kompas.modified', 'События.Закачка.КОМПАС.Исправленный', compulsory_reading=True)
        #
        self.EVENTS_UPLOAD_LOTSMAN_NEW = self.eventsManager.get_event('events.upload.lotsman.new', 'События.Закачка.ЛОЦМАН.Новый', compulsory_reading=True)
        self.EVENTS_UPLOAD_LOTSMAN_MODIFIED = self.eventsManager.get_event('events.upload.lotsman.modified', 'События.Закачка.ЛОЦМАН.Исправленный', compulsory_reading=True)

        self.EVENTS_PRODUCTION_READY_2_LAUNCH = self.eventsManager.get_event('events.production.ready2Launch', 'События.Производство.Выполнен расчет готовности к запуску', compulsory_reading=False)

        self.EVENTS_PRODUCTION_MAKE_ROUTING = self.eventsManager.get_event('events.planing.route.make', 'События.Планирование.Маршрутизация.Формирование', compulsory_reading=False)
        self.EVENTS_PRODUCTION_DELETE_ROUTING = self.eventsManager.get_event('events.planing.route.delete', 'События.Планирование.Маршрутизация.Удаление', compulsory_reading=False)
        self.EVENTS_PLANING_CALC_TIMING = self.eventsManager.get_event('events.planing.timing.calc', 'События.Планирование.Раписание.Расчет', compulsory_reading=False)

        self.EVENTS_PRODUCTION_MAKE_LAUNCH = self.eventsManager.get_event('events.production.launch.make', 'События.Производство.Запуск.Формирование', compulsory_reading=True)
        # self.EVENTS_PRODUCTION_RE_MAKE_LAUNCH = self.eventsManager.get_event('events.production.launch.remake', 'События.Производство.Запуск.Переформирование', compulsory_reading=True)
        self.EVENTS_PRODUCTION_DELETE_LAUNCH = self.eventsManager.get_event('events.production.launch.delete', 'События.Производство.Запуск.Удаление', compulsory_reading=True)
        self.EVENTS_PRODUCTION_CHANGE_LAUNCH = self.eventsManager.get_event('events.production.launch.chancge_status', 'События.Производство.Запуск.Изменение статуса', compulsory_reading=True)
        # self.EVENTS_PRODUCTION_CHANGE_ITEM = self.eventsManager.get_event('events.production.launch.chancge_item', 'События.Производство.Запуск.Изменение товарной позиции', compulsory_reading=True)
        # self.EVENTS_PRODUCTION_ADD_ITEM = self.eventsManager.get_event('events.production.launch.add_item', 'События.Производство.Запуск.Добавление товарной позиции', compulsory_reading=True)
        # self.EVENTS_PRODUCTION_DELETE_ITEM = self.eventsManager.get_event('events.production.launch.delete_item', 'События.Производство.Запуск.Удаление товарной позиции', compulsory_reading=True)



import logging
import os

from django.db import transaction
from django.db.models import TextField

from crypto.models.crypto_file import CryptoManager, Crypto_file
from isc_common.fields.related import ForeignKeyProtect
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Documents_thumbManager(CryptoManager):
    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path": record.file_name,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
            "file_document_thumb_url": f'/logic/DocumentsThumb/Download/{record.id}/',

        }
        return res

    def deleteFromRequest(self, request, removed=None, ):
        res = 0
        ids = request.GET.getlist('ids')

        with transaction.atomic():
            for i in range(0, len(ids), 2):
                id = ids[i]
                mode = ids[i + 1]

                if mode == 'hide':
                    for item in super().filter(id=id):
                        item.soft_delete()
                        Documents_thumb10.objects.filter(path=item.path).soft_delete()
                        res += 1
                else:
                    for item in super().filter(id=id):
                        item.delete()
                        Documents_thumb10.objects.filter(path=item.path).delete()
                        res += 1
        return res


class Documents_thumb(Crypto_file):
    # Менять на cascade нельзя, потому как не происходит удаленеи файлов изображений при удалении документа
    document = ForeignKeyProtect(Documents, verbose_name='КД', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Лоцман', null=True, blank=True)
    path = TextField()

    @property
    def file_name(self):
        dir, filename = os.path.split(self.path)
        return filename

    objects = Documents_thumbManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'JPEG варианты документов уменьшенная копия'



import logging

from django.db.models import TextField, DecimalField, PositiveIntegerField, BooleanField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.status_operation_types import Status_operation_types

logger = logging.getLogger(__name__)


class Operations_viewQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'lastmodified': record.lastmodified,
            'opertype_id': record.opertype.id,
            'opertype__full_name': record.opertype.full_name,
            'status_id': record.status.id if record.status else None,
            'status__code': record.status.code if record.status else None,
            'status__name': record.status.name if record.status else None,
            'location_id': record.location.id if record.location else None,
            'location__code': record.location.code if record.location else None,
            'location__name': record.location.name if record.location else None,
            'location__full_name': record.location.full_name if record.location else None,
            'item_id': record.item.id if record.item else None,
            'value': record.value,
            'parent_id': record.parent_id,
            'isFolder': record.isFolder,
            'description': record.description,
            'item__STMP_1_id': record.item.STMP_1.id if record.item and record.item.STMP_1 and record.item.STMP_1 else None,
            'item__STMP_1__value_str': record.item.STMP_1.value_str if record.item and record.item.STMP_1 and record.item.STMP_1 else None,
            'item__STMP_2_id': record.item.STMP_2.id if record.item and record.item.STMP_2 and record.item.STMP_2 else None,
            'item__STMP_2__value_str': record.item.STMP_2.value_str if record.item and record.item.STMP_2 and record.item.STMP_2 else None,
            'edizm_id': record.edizm.id if record.edizm else None,
            'edizm__code': record.edizm.code if record.edizm else None,
            'edizm__name': record.edizm.name if record.edizm else None,
        }
        return res

    def get_queryset(self):
        return Operations_viewQuerySet(self.model, using=self._db)


class Operations_view(AuditModel):
    opertype = ForeignKeyProtect(Operation_types)
    status = ForeignKeyProtect(Status_operation_types, null=True, blank=True)
    item = ForeignKeyProtect(Item, null=True, blank=True)
    location = ForeignKeyProtect(Locations, null=True, blank=True)
    edizm = ForeignKeyProtect(Ed_izm, null=True, blank=True)
    value = DecimalField(max_digits=19, decimal_places=4, null=True, blank=True)
    description = TextField(null=True, blank=True)
    parent_id = PositiveIntegerField(null=True, blank=True)
    isFolder = BooleanField()

    objects = Operations_viewManager()

    def __str__(self):
        return f"ID:{self.id}, opertype: [{self.opertype}], status: [{self.status}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Опреации системные'
        managed = False
        db_table = 'planing_operations_view'



# Generated by Django 3.0.2 on 2020-01-20 22:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0070_operations_view'),
    ]

    operations = [
        migrations.CreateModel(
            name='Posting_items',
            fields=[
            ],
            options={
                'verbose_name': 'Оприходования',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('planing.operations',),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 20:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0148_launch_operation_resources_q_paralel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='q_paralel',
            field=models.PositiveIntegerField(),
        ),
    ]



import logging

from isc_common.fields.name_field import NameStrictField
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRefHierarcy

logger = logging.getLogger(__name__)


class OperationsQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class OperationsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'full_name': record.full_name,
            'description': record.description,
            'parent_id': record.parent_id,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return OperationsQuerySet(self.model, using=self._db)


class Operations(BaseRefHierarcy):
    name = NameStrictField()
    objects = OperationsManager()

    def __str__(self):
        return f'ID={self.id}, code={self.code}, name={self.name}, description={self.description}'

    class Meta:
        verbose_name = 'Операции'



import logging

from bitfield import BitField
from django.db import transaction, connection
from django.db.models import BigIntegerField
from tqdm import tqdm

from isc_common import Stack
from isc_common.common.mat_views import create_tmp_mat_view
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditQuerySet, AuditModel
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.documents import Documents, DocumentManager

logger = logging.getLogger(__name__)


class Lotsman_documents_hierarcyQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Lotsman_documents_hierarcyManager(AuditManager):
    @staticmethod
    def get_props():
        return BitField(flags=(
            ('relevant', 'Актуальность'),
            ('beenItemed', 'Был внесен в состав изделий'),
        ), default=1, db_index=True)

    @staticmethod
    def make_lines(parent, items_pairs, logger, mat_view_name):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_line import Item_lineManager

        if not isinstance(parent, Item):
            raise Exception(f'item mut be Item instnce.')

        for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'select * from {mat_view_name} where parent_id=%s order by level', [parent.lotsman_document.id]):
            item, _ = Lotsman_documents_hierarcyManager.get_item(lotsman_documents_hierarcy, 'id', items_pairs, logger)

            if lotsman_documents_hierarcy.attr_name != 'Материал':

                if lotsman_documents_hierarcy.section == None:
                    if lotsman_documents_hierarcy.attr_name in ['Чертеж', 'Спецификация']:
                        lotsman_documents_hierarcy.section = 'Документация'
                    elif lotsman_documents_hierarcy.attr_name in ['Сборочная единица']:
                        lotsman_documents_hierarcy.section = 'Сборочные единицы'
                    elif lotsman_documents_hierarcy.attr_name in ['Деталь']:
                        lotsman_documents_hierarcy.section = 'Детали'

                if lotsman_documents_hierarcy.section == None:
                    raise Exception(f'\n\nNot Founf section in \n {lotsman_documents_hierarcy}')

                Документ_на_материал = None
                Наименование_материала = None
                Документ_на_сортамент = None
                Наименование_сортамента = None

                for lotsman_documents_hierarcy_view in Lotsman_documents_hierarcy_view.objects.filter(parent_id=lotsman_documents_hierarcy.id):
                    if lotsman_documents_hierarcy_view.attr_name == 'Материал':
                        Документ_на_материал = lotsman_documents_hierarcy_view.Документ_на_материал
                        Наименование_материала = lotsman_documents_hierarcy_view.Наименование_материала
                        Документ_на_сортамент = lotsman_documents_hierarcy_view.Документ_на_сортамент
                        Наименование_сортамента = lotsman_documents_hierarcy_view.Наименование_сортамента
                        break

                item_line, created = Item_line.objects.update_or_create(
                    parent=parent,
                    child=item,
                    defaults=dict(
                        SPC_CLM_FORMAT=lotsman_documents_hierarcy.SPC_CLM_FORMAT,
                        SPC_CLM_ZONE=lotsman_documents_hierarcy.SPC_CLM_ZONE,
                        SPC_CLM_POS=lotsman_documents_hierarcy.SPC_CLM_POS,
                        SPC_CLM_MARK=lotsman_documents_hierarcy.SPC_CLM_MARK,
                        SPC_CLM_NAME=lotsman_documents_hierarcy.SPC_CLM_NAME,
                        SPC_CLM_COUNT=lotsman_documents_hierarcy.SPC_CLM_COUNT,
                        SPC_CLM_NOTE=lotsman_documents_hierarcy.SPC_CLM_NOTE,
                        SPC_CLM_MASSA=lotsman_documents_hierarcy.SPC_CLM_MASSA,
                        SPC_CLM_MATERIAL=lotsman_documents_hierarcy.SPC_CLM_MATERIAL if lotsman_documents_hierarcy.SPC_CLM_MATERIAL else Наименование_материала,
                        SPC_CLM_USER=lotsman_documents_hierarcy.SPC_CLM_USER,
                        SPC_CLM_KOD=lotsman_documents_hierarcy.SPC_CLM_KOD,
                        SPC_CLM_FACTORY=lotsman_documents_hierarcy.SPC_CLM_FACTORY,
                        Документ_на_материал=Документ_на_материал,
                        Наименование_материала=Наименование_материала,
                        Документ_на_сортамент=Документ_на_сортамент,
                        Наименование_сортамента=Наименование_сортамента,
                        section=lotsman_documents_hierarcy.section,
                        section_num=Item_lineManager.section_num(lotsman_documents_hierarcy.section),
                        subsection=lotsman_documents_hierarcy.subsection,
                    )
                )

                if logger and created:
                    logger.logging(f'\nAdded line: {item_line}', 'debug')

    @staticmethod
    def get_item(lotsman_documents_hierarcy, attribute, items_pairs, logger):
        from kaf_pas.ckk.models.item import Item

        items = [item[1] for item in items_pairs.stack if item[0] == lotsman_documents_hierarcy.__getattribute__(attribute)]
        if len(items) == 0:
            STMP_1 = lotsman_documents_hierarcy.SPC_CLM_NAME
            STMP_2 = lotsman_documents_hierarcy.SPC_CLM_MARK

            item, created = Item.objects.get_or_create(
                STMP_1=STMP_1,
                STMP_2=STMP_2,
                props=Item.props.relevant | Item.props.from_lotsman,
                version=lotsman_documents_hierarcy._Version.value_int,
                defaults=dict(
                    lotsman_document_id=lotsman_documents_hierarcy.id,
                )
            )
            items_pairs.push((lotsman_documents_hierarcy.id, item))

            DocumentManager.link_image_to_lotsman_item(item, logger)

            if logger and created:
                logger.logging(f'\nAdded parent: {item}', 'debug')

            return item, created
        elif len(items) == 1:
            return items[0], False
        else:
            raise Exception(f'Неоднозначный выбор.')

    @staticmethod
    def make_items(logger, owner=None):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.document_attributes import Document_attributesManager
        from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view

        items_pairs = Stack()

        with transaction.atomic():
            imp_frpm_lotsman_label = 'Импорт из Лоцмана'

            parent, created = Item.objects.get_or_create(
                STMP_1=Document_attributesManager.get_or_create_attribute(
                    attr_codes='STMP_1',
                    value_str=imp_frpm_lotsman_label,
                    logger=logger
                ),
                props=Item.props.relevant | Item.props.from_lotsman
            )

            from kaf_pas.system.models.contants import Contants
            item_top_level, _ = Contants.objects.update_or_create(code='top_level', defaults=dict(name='Вершины товарных позиций'))
            const4, _ = Contants.objects.update_or_create(code='lotsman_top_level', defaults=dict(parent=item_top_level, name=imp_frpm_lotsman_label, value=parent.id))

            Item_refs.objects.get_or_create(
                child=parent
            )

            sql_str = '''WITH RECURSIVE r AS (
                                    SELECT *, 1 AS level
                                    FROM kd_lotsman_documents_hierarcy_mview
                                    WHERE parent_id IS NULL
                                          and  props=1  

                                    union all

                                    SELECT kd_lotsman_documents_hierarcy_mview.*, r.level + 1 AS level
                                    FROM kd_lotsman_documents_hierarcy_mview
                                        JOIN r
                                    ON kd_lotsman_documents_hierarcy_mview.parent_id = r.id)

                                select * from r where props=1 order by level'''

            mat_view_name = create_tmp_mat_view(sql_str=sql_str, indexes=['attr_name', 'parent_id'])

            cnt = 0
            with connection.cursor() as cursor:
                cursor.execute(f'select count(*) from {mat_view_name}')
                _cnt, = cursor.fetchone()
                cnt += _cnt

                cursor.execute(f'select count(*) from {mat_view_name} where attr_name=%s', ['''Сборочная единица'''])
                _cnt, = cursor.fetchone()
                cnt += _cnt

            if cnt > 0:
                # FOR DEBUG
                if owner == None:
                    pbar = tqdm(total=cnt)
                # END FOR DEBUG

                if owner != None:
                    owner.cnt += cnt

                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'select * from {mat_view_name} order by level'):

                    if lotsman_documents_hierarcy.attr_name.lower() != 'Материал'.lower():
                        item, created1 = Lotsman_documents_hierarcyManager.get_item(
                            lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                            attribute='id',
                            items_pairs=items_pairs,
                            logger=logger
                        )

                        if lotsman_documents_hierarcy.parent_id != None:
                            parent, created1 = Lotsman_documents_hierarcyManager.get_item(
                                lotsman_documents_hierarcy=lotsman_documents_hierarcy,
                                attribute='parent_id',
                                items_pairs=items_pairs,
                                logger=logger
                            )

                        if parent != item:
                            item_refs, created2 = Item_refs.objects.get_or_create(parent=parent, child=item)

                        # FOR DEBUG
                        if created == True or created1 == True or created2 == True:
                            pass
                        # END FOR DEBUG

                    if owner != None:
                        owner.pbar_progress()
                    # FOR DEBUG
                    else:
                        pbar.update()
                    # END FOR DEBUG

                for lotsman_documents_hierarcy in Lotsman_documents_hierarcy_view.objects.raw(f'''select * from {mat_view_name} where attr_name=%s order by level''', ['''Сборочная единица''']):

                    item, _ = Lotsman_documents_hierarcyManager.get_item(lotsman_documents_hierarcy, 'id', items_pairs, logger)

                    Lotsman_documents_hierarcyManager.make_lines(
                        parent=item,
                        items_pairs=items_pairs,
                        logger=logger,
                        mat_view_name=mat_view_name
                    )

                    Lotsman_documents_hierarcy.objects.update_or_create(
                        id=lotsman_documents_hierarcy.id,
                        defaults=dict(
                            props=lotsman_documents_hierarcy.props | Lotsman_documents_hierarcy.props.beenItemed
                        ))

                    if owner != None:
                        owner.pbar_progress()
                    # FOR DEBUG
                    else:
                        pbar.update()
                    # END FOR DEBUG

                # FOR DEBUG
                if owner == None:
                    pbar.close()
                # END FOR DEBUG

    @staticmethod
    def make_mview():
        from kaf_pas.system.models.contants import Contants
        from django.db import connection

        fields_sql = []
        sql_array = []

        parent_system_const, _ = Contants.objects.update_or_create(
            code='lotsman_attibutes',
            defaults=dict(name='Атрибуты товарных позиций импортированных из Лоцмана')
        )

        attr_map = {
            'Зона': 'SPC_CLM_ZONE',
            'Код': 'SPC_CLM_KOD',
            'Масса': 'SPC_CLM_MASSA',
            'Материал': 'SPC_CLM_MATERIAL',
            'Наименование': 'SPC_CLM_NAME',
            'Обозначение': 'SPC_CLM_MARK',
            'Позиция': 'SPC_CLM_POS',
            'Пользовательская': 'SPC_CLM_USER',
            'Предприятие - изготовитель': 'SPC_CLM_FACTORY',
            'Примечание': 'SPC_CLM_NOTE',
            'Формат': 'SPC_CLM_FORMAT',
        }

        for name, code in attr_map.items():
            Contants.objects.update_or_create(
                code=code,
                defaults=dict(
                    name=name,
                    parent=parent_system_const
                )
            )

        m_view_name = 'kd_lotsman_documents_hierarcy_mview'
        m_view_recurs_name = 'kd_lotsman_documents_hierarcy_recurs_view'

        sql_array.append(f'DROP VIEW IF EXISTS {m_view_recurs_name} CASCADE')
        sql_array.append(f'DROP MATERIALIZED VIEW IF EXISTS {m_view_name} CASCADE')
        sql_array.append(f'''CREATE MATERIALIZED VIEW {m_view_name} AS SELECT lts.id,
                                                                                   lts.deleted_at,
                                                                                   lts.editing,
                                                                                   lts.deliting,
                                                                                   lts.lastmodified,                                                                                
                                                                                   ltsr.parent_id,
                                                                                   lts.props,
                                                                                   lts.attr_type_id,
                                                                                   lts.document_id,
                                                                                   CASE
                                                                                        WHEN (select count(1) as count
                                                                                              from kd_lotsman_documents_hierarcy_refs hr                                                            	
                                                                                              where hr.parent_id = lts.id) > 0 THEN true
                                                                                        ELSE false
                                                                                   END AS "isFolder",                                                                                 
                                                                                   ltsr.section, 
                                                                                   ltsr.subsection,
                                                                                   att.code attr_code,
                                                                                   att.name attr_name
                                                                                   $COMMA
                                                                                   $FIELDS     
                                                                            FROM kd_lotsman_documents_hierarcy lts
                                                                                    join kd_lotsman_documents_hierarcy_refs ltsr on ltsr.child_id = lts.id
                                                                                    join ckk_attr_type att on att.id = lts.attr_type_id  WITH DATA''')

        for field in Contants.objects.filter(parent__code='lotsman_attibutes'):
            fields_sql.append(f'''( SELECT kat.id
                                               FROM kd_document_attributes kat
                                                 JOIN kd_lotsman_document_attr_cross dc ON kat.id = dc.attribute_id
                                                 JOIN ckk_attr_type att ON att.id = kat.attr_type_id
                                              WHERE dc.document_id = lts.id AND att.code::text = '{field.code}'::text limit 1) AS "{field.code}_id"''')
            # fields_sql.append(f'''( SELECT kat.value_str
            #                                    FROM kd_document_attributes kat
            #                                      JOIN kd_lotsman_document_attr_cross dc ON kat.id = dc.attribute_id
            #                                      JOIN ckk_attr_type att ON att.id = kat.attr_type_id
            #                                   WHERE dc.document_id = lts.id AND att.code::text = '{field.code}'::text limit 1) AS "{field.code}_value_str"''')
            # fields_sql.append(f'''( SELECT kat.value_int
            #                                    FROM kd_document_attributes kat
            #                                      JOIN kd_lotsman_document_attr_cross dc ON kat.id = dc.attribute_id
            #                                      JOIN ckk_attr_type att ON att.id = kat.attr_type_id
            #                                   WHERE dc.document_id = lts.id AND att.code::text = '{field.code}'::text limit 1) AS "{field.code}_value_int"''')
        # sql_array.append(f'REFRESH MATERIALIZED VIEW {m_view_name};')

        if len(fields_sql) > 0:
            sql_str = ';\n'.join(sql_array).replace('$FIELDS', ',\n'.join(fields_sql))
            sql_str = sql_str.replace('$COMMA', ',')
        else:
            sql_str = ';\n'.join(sql_array).replace('$FIELDS', '')
            sql_str = sql_str.replace('$COMMA', '')

        with connection.cursor() as cursor:
            logger.debug(f'\n{sql_str}')
            cursor.execute(sql_str)
            logger.debug(f'{m_view_name} recreated')

            sql_array = []
            sql_array.append(f'''CREATE VIEW {m_view_recurs_name} AS select * from (WITH RECURSIVE r AS (
                                    SELECT *, 1 AS level
                                    FROM {m_view_name}
                                    WHERE parent_id IS NULL
                                
                                    union all
                                
                                    SELECT {m_view_name}.*, r.level + 1 AS level
                                    FROM {m_view_name}
                                             JOIN r
                                                  ON {m_view_name}.parent_id = r.id)                                
                                select * from r order by level) as a''')
            sql_str = ';\n'.join(sql_array)
            logger.debug(f'\n{sql_str}')
            cursor.execute(sql_str)
            logger.debug(f'{m_view_recurs_name} recreated')

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'parent': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Lotsman_documents_hierarcyQuerySet(self.model, using=self._db)

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    qty, _ = super().filter(id=id).delete()
                res += qty
        return res


class Lotsman_documents_hierarcy(AuditModel):
    id = BigIntegerField(primary_key=True, verbose_name="Идентификатор")
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    document = ForeignKeyProtect(Documents)
    props = Lotsman_documents_hierarcyManager.get_props()

    objects = Lotsman_documents_hierarcyManager()

    def __str__(self):
        return f'ID:{self.id}, attr_type: {self.attr_type}, document: {self.document}, props: {self.props}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Иерархия документа из Лоцмана'



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.documents_history import Documents_history, Documents_historyManager


@JsonResponseWithException()
def Documents_history_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Documents_history.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=Documents_historyManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Add(request):
    return JsonResponse(DSResponseAdd(data=Documents_history.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Update(request):
    return JsonResponse(DSResponseUpdate(data=Documents_history.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Documents_history.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Documents_history.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Info(request):
    return JsonResponse(DSResponse(request=request, data=Documents_history.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Documents_history_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Documents_history.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)            



import logging

from django.core.management import BaseCommand

from isc_common.logger.Logger import Logger

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = 'Создание товаоных позиций после импорта из Лоцмана'

    def handle(self, *args, **options):
        try:
            from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcyManager
            from isc_common.management.commands.refresh_mat_views import refresh_mat_view

            Lotsman_documents_hierarcyManager.make_items(logger=logger)
            refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
        except Exception as ex:
            raise ex



from django.urls import path

from kaf_pas.planing.views import operation_locations

urlpatterns = [

    path('Operation_locations/Fetch/', operation_locations.Operation_locations_Fetch),
    path('Operation_locations/Add', operation_locations.Operation_locations_Add),
    path('Operation_locations/Update', operation_locations.Operation_locations_Update),
    path('Operation_locations/Remove', operation_locations.Operation_locations_Remove),
    path('Operation_locations/Lookup/', operation_locations.Operation_locations_Lookup),
    path('Operation_locations/Info/', operation_locations.Operation_locations_Info),
    path('Operation_locations/Copy', operation_locations.Operation_locations_Copy),

]



# Generated by Django 3.0.2 on 2020-01-21 07:44

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0125_auto_20200121_0743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='demand',
            name='precent_item',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='sales.Precent_items'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 10:59

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0013_auto_20190426_1003'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_plan_refs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child_Operations_plan', to='production.OperationsPlan')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent_Operations_plan', to='production.OperationsPlan')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.planing.models.operations import OperationsManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        data = {'id': 17, 'code': 'Тест111/1', 'name': None, 'date': '2020-01-18T00:00:00.000', 'description': None, 'parent_id': None, 'demand_id': 38, 'demand__code': 'Тест111', 'demand__date': '2019-11-13T00:00:00.000', 'status_id': 3, 'status__code': 'formirovanie', 'status__name': 'Формирование', 'qty': 2, 'routing_made': False, 'isFolder': False, 'props': 0, 'editing': True, 'deliting': True, 'user_id': 2}
        OperationsManager.make_routing(data=data)



import logging

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Lotsman_documents_hierarcy_refsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def _check(self, **kwargs):
        if kwargs.get('parent') != None:
            if isinstance(kwargs.get('parent'), Lotsman_documents_hierarcy):
                parent = kwargs.get('parent').id
            else:
                parent = kwargs.get('parent')

            if isinstance(kwargs.get('child'), Lotsman_documents_hierarcy):
                child = kwargs.get('child').id
            else:
                child = kwargs.get('child')

            if parent == child:
                raise Exception(f'Attempt to write circular reference id ({parent})')

    def create(self, **kwargs):
        self._check(**kwargs)
        return super().create(**kwargs)

    def update(self, **kwargs):
        self._check(**kwargs)
        return super().update(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().get_or_create(defaults, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().update_or_create(defaults, **kwargs)


class Lotsman_documents_hierarcy_refsManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Lotsman_documents_hierarcy_refsQuerySet(self.model, using=self._db)


class Lotsman_documents_hierarcy_refs(AuditModel):
    child = ForeignKeyProtect(Lotsman_documents_hierarcy, related_name='Lotsman_documents_hierarcy_child')
    parent = ForeignKeyProtect(Lotsman_documents_hierarcy, related_name='Lotsman_documents_hierarcy_parent', null=True, blank=True)

    section = CodeField(verbose_name="Раздел", null=True, blank=True)
    subsection = CodeField(verbose_name="Подраздел", null=True, blank=True)

    objects = Lotsman_documents_hierarcy_refsManager()
    objects_tree = TreeAuditModelManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('child', 'parent', 'section', 'subsection'),)



# Generated by Django 2.2 on 2019-04-25 11:23

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('ckk', '0038_auto_20190425_0829'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_plan_template_locations',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
            ],
            options={
                'verbose_name': 'tста для операций планирования',
            },
        ),
        migrations.CreateModel(
            name='Operation_plan_templates',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
            ],
            options={
                'verbose_name': 'Шаблоны операций планирования',
            },
        ),
        migrations.CreateModel(
            name='Operation_plan_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('code', isc_common.fields.code_field.CodeField(unique=True)),
            ],
            options={
                'verbose_name': 'Типи операций планирования',
            },
        ),
        migrations.CreateModel(
            name='OperationsPlan',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
            ],
            options={
                'verbose_name': 'Операции планировая',
            },
        ),
        migrations.CreateModel(
            name='Operation_plan_templates_item_lines',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('itemLine', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item_line')),
                ('operation_plan_template', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operation_plan_templates')),
            ],
            options={
                'verbose_name': 'Строки спецификации',
            },
        ),
    ]



import logging
import os
from os.path import getsize

from bitfield import BitField
from django.db import transaction
from django.db.models import DateTimeField, TextField, PositiveIntegerField
from isc_common import delAttr, replace_alt_set, setAttr
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.management.commands.refresh_mat_views import refresh_mat_view
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel

from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.pathes import Pathes

logger = logging.getLogger(__name__)


class DocumentQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    pass


class DocumentManager(CommonManagetWithLookUpFieldsManager):
    type_bad = Attr_type.objects.get_attr(code='KD_BAD')

    def get_queryset(self):
        return DocumentQuerySet(self.model, using=self._db)

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path_id": record.path.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "file_document": record.file_document,
            "file_size": record.file_size,
            "file_modification_time": record.file_modification_time,
            "file_access_time": record.file_access_time,
            "file_change_time": record.file_change_time,
            "lastmodified": record.lastmodified,
            'load_error': f'<pre>{record.relevant}</pre>' if record.relevant else None,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_ex(self, *args, **kwargs):
        file_name = kwargs.get('file_document')
        file_drive = kwargs.get('file_drive')
        virt_path = kwargs.get('virt_path')
        attr_type = kwargs.get('attr_type')
        delAttr(kwargs, 'file_document')

        if virt_path:
            if virt_path[0:1] == os.sep or virt_path[0:1] == os.altsep:
                virt_path = virt_path[1:]

        if virt_path:
            if file_name.find(virt_path) == -1:
                _file_name = f'{virt_path}{os.sep}{file_name}'.replace(os.altsep, os.sep)
            else:
                _file_name = file_name.replace(os.altsep, os.sep)
        else:
            _file_name = file_name.replace(os.altsep, os.sep)

        for doc in Documents.objects.filter(path=kwargs.get('path'), attr_type=attr_type, props=Documents.props.relevant).exclude(attr_type=self.type_bad):
            real_file = replace_alt_set(f'{file_drive}{os.sep}{file_name}')

            if replace_alt_set(doc.file_document) == replace_alt_set(_file_name):
                real_file_size = getsize(real_file.replace(replace_alt_set(virt_path), '') if virt_path else real_file)

                # real_file_modification_time = datetime.fromtimestamp(getmtime(real_file)).replace(microsecond=0)
                # system_file_modification_time = doc.file_modification_time.replace(microsecond=0)

                # real_file_access_time = datetime.fromtimestamp(getatime(real_file)).replace(microsecond=0)
                # system_file_access_time = doc.file_access_time.replace(microsecond=0)
                #
                # real_file_change_time = datetime.fromtimestamp(getctime(real_file)).replace(microsecond=0)
                # system_file_change_time = doc.file_change_time.replace(microsecond=0)

                if doc.file_size != real_file_size:
                    # logger.debug(f'Неравенство размеров {doc.file_size} != {real_file_size}')
                    return doc, True
                # elif system_file_modification_time != real_file_modification_time:
                #     logger.debug(f'Неравенство file_modification_time {system_file_modification_time} != {real_file_modification_time}')
                #     return doc, True
                # elif system_file_access_time != real_file_access_time:
                #     logger.debug(f'Неравенство file_access_time {system_file_access_time} != {real_file_access_time}')
                #     return doc, True
                # elif system_file_change_time != real_file_change_time:
                #     logger.debug(f'Неравенство file_change_time {system_file_change_time} != {real_file_change_time}')
                #     return doc, True
                else:
                    return doc, False
        return None, None

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        relevant = data.get('relevant')
        props = data.get('props')

        if relevant == 'Актуален':
            props |= Documents.props.relevant
        else:
            props &= ~Documents.props.relevant

        res = super().get(id=data.get('id'))
        res.props = props
        res.save()
        refresh_mat_view('kd_documents_mview')

        return data

    @staticmethod
    def delete(id, user):
        from isc_common.auth.models.user import User
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
        from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs

        if not isinstance(user, User):
            raise Exception(f'user  must be a User instance.')

        res = 0

        Uploades_documents.objects.filter(document_id=id).delete()
        Documents_thumb.objects.filter(document_id=id).delete()
        Documents_thumb.objects.filter(document_id=id).delete()
        Documents_thumb10.objects.filter(document_id=id).delete()
        Document_attr_cross.objects.filter(document_id=id).delete()

        for item in Item.objects.filter(document_id=id):
            from kaf_pas.ckk.models.item import ItemManager
            ItemManager.delete_recursive(item=item, user=user)

        Lotsman_documents_hierarcy_files.objects.filter(lotsman_document__document_id=id).delete()

        Documents_thumb.objects.filter(lotsman_document__document_id=id).delete()
        Documents_thumb10.objects.filter(lotsman_document__document_id=id).delete()

        Documents_thumb.objects.filter(document_id=id).delete()
        Documents_thumb10.objects.filter(document_id=id).delete()

        for lotsman_documents_hierarcy in Lotsman_documents_hierarcy.objects.filter(document_id=id):
            Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_documents_hierarcy).delete()
            Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_documents_hierarcy).delete()
            lotsman_documents_hierarcy.delete()

        res += Documents.objects.filter(id=id).delete()[0]
        return res

    def deleteFromRequest(self, request):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from isc_common.auth.models.user import User

        ids = request.GET.getlist('ids')
        user_id = int(request.GET.get('user_id'))

        res = 0
        with transaction.atomic():
            for i in range(0, len(ids), 2):
                id = ids[i]
                visibleMode = ids[i + 1]

                if visibleMode != "none":
                    for item in Item.objects.filter(document_id=id):
                        # Item_refs.objects.filter(child=item).soft_delete(visibleMode=visibleMode)
                        # Item_refs.objects.filter(parent=item).soft_delete(visibleMode=visibleMode)

                        # Item_line.objects.filter(child=item).soft_delete(visibleMode=visibleMode)
                        # Item_line.objects.filter(parent=item).soft_delete(visibleMode=visibleMode)

                        Item_image_refs.objects.filter(item=item).soft_delete(visibleMode=visibleMode)

                        Documents_thumb.objects.filter(lotsman_document__document_id=id).soft_delete(visibleMode=visibleMode)
                        Documents_thumb10.objects.filter(lotsman_document__document_id=id).soft_delete(visibleMode=visibleMode)

                        Documents_thumb.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)
                        Documents_thumb10.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)

                    Item.objects.filter(document_id=id).soft_delete(visibleMode=visibleMode)
                    res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
                else:
                    res += DocumentManager.delete(id, User.objects.get(id=user_id))
        refresh_mat_view('kd_documents_mview')
        refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
        return res

    @staticmethod
    def rec_image_cwd(item_id, STMP_2, logger):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.ckk.models.item import Item
        for item_image in Item_image_refs.objects.select_related('item').filter(item__props__in=[Item.props.relevant | Item.props.from_cdw, Item.props.relevant | Item.props.from_pdf], item__STMP_2__value_str=STMP_2):
            _, created = Item_image_refs.objects.get_or_create(item_id=item_id, thumb_id=item_image.thumb_id, thumb10_id=item_image.thumb10_id)
            # if created:
            #     if logger:
            #         logger.logging(f'\nAdded: {item_image}', 'debug')

    @staticmethod
    def del_blancks1(STMP_2):
        STMP_2_1 = ' '.join(STMP_2.split())
        if STMP_2_1 != STMP_2:
            return STMP_2_1
        return STMP_2

    @staticmethod
    def link_image_to_item(item, logger):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

        document = item.document
        if document != None:
            for document_thumb in Documents_thumb.objects.filter(document=document):
                item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb=document_thumb)
                # if logger:
                #     logger.logging(f'\nItem_image_refs: {item_image_refs}, created: {created}', 'debug')

            for document_thumb10 in Documents_thumb10.objects.filter(document=document):
                item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb10=document_thumb10)
                # if logger:
                #     logger.logging(f'\nItem_image_refs: {item_image_refs}, created: {created}', 'debug')

    @staticmethod
    def link_image_to_lotsman_item(item, logger):
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10

        lotsman_document = item.lotsman_document
        if lotsman_document != None:
            for document_thumb in Documents_thumb.objects.filter(lotsman_document=lotsman_document):
                item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb=document_thumb)
                if logger:
                    logger.logging(f'\nItem_image_refs: {item_image_refs}, created: {created}', 'debug')

            for document_thumb10 in Documents_thumb10.objects.filter(lotsman_document=lotsman_document):
                item_image_refs, created = Item_image_refs.objects.get_or_create(item=item, thumb10=document_thumb10)
                if logger:
                    logger.logging(f'\nItem_image_refs: {item_image_refs}, created: {created}', 'debug')


    @staticmethod
    def make_spw(document, logger=None):
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.document_attributes import Document_attributes
        from kaf_pas.kd.models.spw_attrs import Spw_attrs
        from kaf_pas.kd.models.spw_attrs import Spw_attrsQuerySet
        from kaf_pas.system.models.contants import Contants

        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:

                STMP_1 = Spw_attrs.objects.all().get_attr(document=document, code='STMP_1')
                STMP_2 = Spw_attrs.objects.all().get_attr(document=document, code='STMP_2')

                if STMP_1 != None or STMP_2 != None:
                    kwargs = dict(
                        STMP_1_id=STMP_1.id if STMP_1 else None,
                        STMP_2_id=STMP_2.id if STMP_2 else None,
                        props=Item.props.relevant | Item.props.from_spw
                    )

                    parent, created = Item.objects.get_or_create(
                        **kwargs,
                        defaults=dict(document=document))

                    if created:
                        item_refs = Item_refs.objects.create(parent_id=int(top_level.value), child=parent)
                        # if logger:
                        #     logger.logging(f'\nAdded: {item_refs} as tpo level', 'debug')

                    if STMP_2 != None:
                        DocumentManager.rec_image_cwd(parent.id, STMP_2.value_str, logger)
                    DocumentManager.link_image_to_item(parent, logger)

                    query_spw = Spw_attrs.objects.filter(document=document).order_by(*['position_in_document'])
                    specification = Spw_attrsQuerySet.make_specification(queryResult=query_spw)

                    SPC_CLM_MARK_old = None
                    for line_specification in specification:
                        SPC_CLM_NAME = line_specification.get('SPC_CLM_NAME')
                        SPC_CLM_NAME_ID = line_specification.get('SPC_CLM_NAME_ID')

                        SPC_CLM_MARK = line_specification.get('SPC_CLM_MARK')
                        SPC_CLM_MARK_ID = line_specification.get('SPC_CLM_MARK_ID')

                        if SPC_CLM_NAME_ID != None or SPC_CLM_MARK_ID != None:
                            if isinstance(SPC_CLM_MARK, str) and SPC_CLM_MARK.startswith('-') and isinstance(SPC_CLM_MARK_ID, int):
                                _SPC_CLM_MARK = f'{SPC_CLM_MARK_old}{SPC_CLM_MARK}'
                                for document_attribute in Document_attributes.objects.filter(id=SPC_CLM_MARK_ID):
                                    try:
                                        _document_attribute = Document_attributes.objects.get(attr_type=document_attribute.attr_type, value_str=_SPC_CLM_MARK)
                                        Document_attr_cross.objects. \
                                            filter(attribute=document_attribute). \
                                            update(
                                            attribute=_document_attribute,
                                            document=document
                                        )
                                        setAttr(line_specification, 'SPC_CLM_MARK_ID', _document_attribute.id)
                                        # try:
                                        #     document_attribute.delete()
                                        # except Exception as ex:
                                        #     self.logging(ex, 'warning')
                                    except Document_attributes.DoesNotExist:
                                        document_attribute.value_str = _SPC_CLM_MARK
                                        document_attribute.save()
                                        if logger:
                                            logger.logging(f'=================================================>>>>>> Updated: {_SPC_CLM_MARK}')

                            elif isinstance(SPC_CLM_MARK, str):
                                SPC_CLM_MARK_old = SPC_CLM_MARK

                            child_args = dict(
                                STMP_1=SPC_CLM_NAME_ID,
                                STMP_2=SPC_CLM_MARK_ID,
                            )

                            _props = Item.props.relevant | Item.props.from_spw | Item.props.for_line

                            childs = list(Item.objects.filter(
                                STMP_1_id=child_args.get('STMP_1'),
                                STMP_2_id=child_args.get('STMP_2'),
                                props=_props,
                            ).order_by(*['version']))

                            if len(childs) > 1:
                                child, created = childs[0], False
                            else:
                                child, created = Item.objects.get_or_create(
                                    STMP_1_id=child_args.get('STMP_1'),
                                    STMP_2_id=child_args.get('STMP_2'),
                                    props=_props,
                                    defaults=dict(document=document)
                                )

                            if created:
                                if child.STMP_2 != None:
                                    DocumentManager.rec_image_cwd(child.id, child.STMP_2.value_str, logger)
                                # if logger:
                                #     logger.logging(f'\nAdded: {child} as child', 'debug')
                            # else:
                            #     if logger:
                            #         logger.logging(f'\nNot Added: {child} as child', 'debug')

                            item_refs, created = Item_refs.objects.get_or_create(parent=parent, child=child)
                            # if created:
                            #     if logger:
                            #         logger.logging(f'\nAdded: {item_refs}', 'debug')
                            # else:
                            #     if logger:
                            #         logger.logging(f'\nNot Added: {item_refs}', 'debug')

                            defaults = dict(
                                parent=parent,
                                child=child,
                                SPC_CLM_FORMAT_id=line_specification.get('SPC_CLM_FORMAT_ID'),
                                SPC_CLM_ZONE_id=line_specification.get('SPC_CLM_ZONE_ID'),
                                SPC_CLM_POS_id=line_specification.get('SPC_CLM_POS_ID'),
                                SPC_CLM_MARK_id=line_specification.get('SPC_CLM_MARK_ID'),
                                SPC_CLM_NAME_id=line_specification.get('SPC_CLM_NAME_ID'),
                                SPC_CLM_COUNT_id=line_specification.get('SPC_CLM_COUNT_ID'),
                                SPC_CLM_NOTE_id=line_specification.get('SPC_CLM_NOTE_ID'),
                                SPC_CLM_MASSA_id=line_specification.get('SPC_CLM_MASSA_ID'),
                                SPC_CLM_MATERIAL_id=line_specification.get('SPC_CLM_MATERIAL_ID'),
                                SPC_CLM_USER_id=line_specification.get('SPC_CLM_USER_ID'),
                                SPC_CLM_KOD_id=line_specification.get('SPC_CLM_KOD_ID'),
                                SPC_CLM_FACTORY_id=line_specification.get('SPC_CLM_FACTORY_ID'),

                                section=line_specification.get('section'),
                                subsection=line_specification.get('subsection'),
                            )

                            item_line, created = Item_line.objects.get_or_create(parent=parent, child=child, defaults=defaults)
                            # if created:
                            #     if logger:
                            #         logger.logging(f'\nAdded: {item_line} as item_line', 'debug')
                            # else:
                            #     if logger:
                            #         logger.logging(f'\nNot Added: {item_line} as item_line', 'debug')

                else:
                    # if logger:
                    #     logger.logging(f'\nSTMP_1 and STMP_2 is None, document: {document}', 'debug')
                    document.delete_soft()

            document.props |= Documents.props.beenItemed
            document.save()

    @staticmethod
    def make_cdw(document, logger=None):
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.ckk.models.item import Item

        from kaf_pas.system.models.contants import Contants
        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:
                from kaf_pas.kd.models.cdw_attrs import Cdw_attrs
                STMP_1 = Cdw_attrs.objects.all().get_attr(document=document, code='STMP_1')
                STMP_2 = Cdw_attrs.objects.all().get_attr(document=document, code='STMP_2')

                if STMP_1 != None or STMP_2 != None:
                    args = dict(
                        STMP_1_id=STMP_1.id if STMP_1 else None,
                        STMP_2_id=STMP_2.id if STMP_2 else None,
                        props=Item.props.relevant | Item.props.from_cdw
                    )

                    cnt = 0
                    items = []

                    for item in Item.objects.filter(**args):

                        dir, file_name = os.path.split(document.file_document)
                        dir1, file_name1 = os.path.split(item.document.file_document)
                        if file_name == file_name1 and document.file_size == item.document.file_size:
                            items.append(item)
                            cnt += 1

                    if cnt == 0:
                        item, created = Item.objects.get_or_create(**args, defaults=dict(document=document))

                        # if created:
                        #     if logger:
                        #         logger.logging(f'\nAdded: {item}, cdw', 'debug')
                        # else:
                        #     if logger:
                        #         logger.logging('==============================================================================================================', 'debug')

                        try:
                            Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                        except Item_refs.MultipleObjectsReturned:
                            ...

                        DocumentManager.link_image_to_item(item, logger)
                    else:
                        for item in items:
                            # if logger:
                            #     logger.logging(f'\nНайдено: элемент пришедший из {cnt + 1}-x источников : {item}', 'debug')
                            DocumentManager.link_image_to_item(item, logger)

                else:
                    # if logger:
                    #     logger.logging(f'\nSTMP_1 is None and STMP_2 is None, document: {document}', 'debug')
                    document.delete_soft()

                document.props |= Documents.props.beenItemed
                document.save()

    @staticmethod
    def make_pdf(document, STMP_1_type, STMP_2_type, logger=None):
        from kaf_pas.kd.models.document_attributes import Document_attributes
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.system.models.contants import Contants

        top_level = Contants.objects.get(code='audo_top_level')

        with transaction.atomic():
            if document.file_document.lower().find('мусор') == -1:
                full_path = document.file_document
                _, file_name = os.path.split(full_path)
                file_name_part = file_name.split(' - ')
                if len(file_name_part) == 2:

                    STMP_1, ext = os.path.splitext(file_name_part[1].strip())
                    STMP_2 = DocumentManager.del_blancks1(file_name_part[0].strip())
                else:
                    STMP_1 = str(file_name.strip()).replace('.pdf', '').replace('.PDF', '')
                    STMP_2 = STMP_1

                args = dict(
                    STMP_1=STMP_1,
                    STMP_2=STMP_2,
                )

                cnt = 0
                items = []
                from kaf_pas.ckk.models.item import Item
                for item in Item.objects.filter(
                        STMP_1__value_str=args.get('STMP_1'),
                        STMP_2__value_str=args.get('STMP_2'),
                        props=Item.props.relevant | Item.props.from_pdf
                ):

                    dir, file_name = os.path.split(document.file_document)
                    dir1, file_name1 = os.path.split(item.document.file_document)
                    if file_name == file_name1:
                        items.append(item)
                        cnt += 1

                if cnt == 0:
                    STMP_1_attr, _ = Document_attributes.objects.get_or_create(value_str=STMP_1, attr_type=STMP_1_type)
                    STMP_2_attr, _ = Document_attributes.objects.get_or_create(value_str=STMP_2, attr_type=STMP_2_type)

                    item, created = Item.objects.get_or_create(
                        STMP_1=STMP_1_attr,
                        STMP_2=STMP_2_attr,
                        document=document,
                        props=Item.props.relevant | Item.props.from_pdf,
                        defaults=dict(document=document)
                    )

                    # if created:
                    #     if logger:
                    #         logger.logging(f'\nAdded: {item}', 'debug')
                    # else:
                    #     if logger:
                    #         logger.logging('==============================================================================================================', 'debug')

                    try:
                        Item_refs.objects.get_or_create(parent_id=int(top_level.value), child=item)
                    except Item_refs.MultipleObjectsReturned:
                        ...

                    DocumentManager.link_image_to_item(item, logger)
                else:
                    for item in items:
                        # if logger:
                        #     logger.logging(f'\nНайдено: элемент пришедший из {cnt + 1}-x источников : {item}', 'debug')
                        DocumentManager.link_image_to_item(item, logger)

            document.props |= Documents.props.beenItemed
            document.save()


class Documents(AuditModel):
    path = ForeignKeyProtect(Pathes, verbose_name='Путь к документу')
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    file_document = TextField(verbose_name='Полный путь к файлу', db_index=True)
    file_size = PositiveIntegerField(verbose_name='Размер файла')
    file_modification_time = DateTimeField(verbose_name='Дата время поcледнего модификации документа', null=True, blank=True, db_index=True)
    file_access_time = DateTimeField(verbose_name='Дата время поcледнего доступа к документу', null=True, blank=True, db_index=True)
    file_change_time = DateTimeField(verbose_name='Дата время изменнения документа', null=True, blank=True, db_index=True)
    props = BitField(flags=(
        ('relevant', 'Актуальность'),
        ('beenItemed', 'Был внесен в состав изделий'),
    ), default=1, db_index=True)
    load_error = TextField(null=True, blank=True)

    @property
    def file_name(self):
        (_, filename) = os.path.split(self.file_document)
        return filename

    objects = DocumentManager()

    @property
    def is_cdw(self):
        return self.attr_type.code == 'CDW'

    @property
    def is_spw(self):
        return self.attr_type.code == 'CPW'

    @property
    def is_pdf(self):
        dir, file = os.path.split(self.file_document)
        _, ext = os.path.splitext(file)
        if ext.lower() == '.pdf':
            return True
        else:
            return False

    def __str__(self):
        return f"ID: {self.id}, " \
               f"{self.file_document}, " \
               f"attr_type: [{self.attr_type}], " \
               f"file_document: {self.file_document}, " \
               f"file_size: {self.file_size}, " \
               f"file_modification_time: {self.file_modification_time}, " \
               f"file_access_time: {self.file_access_time}, " \
               f"file_change_time: {self.file_change_time}, " \
               f"props: {self.props}, " \
               f"load_error: {self.load_error}"

    class Meta:
        verbose_name = 'Конструкторские документы'



from django.utils.translation import ugettext_lazy as _

import logging

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from kaf_pas.planing.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_refsQuerySet(AuditQuerySet):
    def delete(self):
        return super().delete()

    def _check(self, **kwargs):
        if kwargs.get('income') != None:
            if isinstance(kwargs.get('income'), Operations):
                income = kwargs.get('income').id
            else:
                income = kwargs.get('income')

            if isinstance(kwargs.get('outcome'), Operations):
                outcome = kwargs.get('outcome').id
            else:
                outcome = kwargs.get('outcome')

            if income == outcome:
                raise Exception(f'Attempt to write circular reference id ({income})')

    def create(self, **kwargs):
        self._check(**kwargs)
        return super().create(**kwargs)

    def update(self, **kwargs):
        self._check(**kwargs)
        return super().update(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().get_or_create(defaults, **kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        return super().update_or_create(defaults, **kwargs)


class Operation_refsManager(TreeAuditModelManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'income': record.income.id if record.income else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_refsQuerySet(self.model, using=self._db)


class Operation_refs(AuditModel):
    parent = ForeignKeyProtect(Operations, related_name='operation_parent', blank=True, null=True)
    child = ForeignKeyProtect(Operations, related_name='operation_child')

    objects = Operation_refsManager()

    def __str__(self):
        return f"ID:{self.id}"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица операций планирования'



# Generated by Django 2.2.6 on 2019-10-25 13:11

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0094_auto_20191024_1218'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent_items',
            name='precent_item_type',
        ),
        migrations.AddField(
            model_name='precent',
            name='precent_type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent_item_types'),
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.ckk.models.item_view import Item_view
from kaf_pas.kd.models.document_attributes import Document_attributes

logger = logging.getLogger(__name__)
logger1 = logging.getLogger(f'{__name__}_1')


class Command(BaseCommand):
    help = "Перенос чертежей в подпапку"

    def handle(self, *args, **options):

        logger.info(self.help)

        self.pbar = tqdm(total=Item_view.objects.filter(parent_id__isnull=True, props__in=[Item.props.relevant | Item.props.from_cdw, Item.props.relevant | Item.props.from_pdf]).count())
        cdws_type, _ = Attr_type.objects.get_or_create(code='CDWS', defaults=dict(name='Чертежи'))
        cdws_attr, _ = Document_attributes.objects.get_or_create(attr_type=cdws_type, value_str='Автоматически закачанные чертежи')
        cdws_item, _ = Item.objects.get_or_create(STMP_1=cdws_attr, props=Item.props.relevant | Item.props.from_cdw)
        item_refs, _ = Item_refs.objects.get_or_create(child=cdws_item)

        with transaction.atomic():
            for item in Item_view.objects.filter(parent_id__isnull=True, props__in=[Item.props.relevant | Item.props.from_cdw, Item.props.relevant | Item.props.from_pdf]):
                res = Item_refs.objects.filter(parent_id__isnull=True, child_id=item.id).delete()
                item_refs, _ = Item_refs.objects.get_or_create(child_id=item.id, parent=cdws_item)

                if self.pbar:
                    self.pbar.update(1)

        if self.pbar:
            self.pbar.close()

        logger.info("Загрузка выполнена.")
        # </editor-fold>

# with transaction.atomic():



# Generated by Django 2.2.6 on 2019-10-20 18:38

import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('ckk', '0158_delete_status'),
        ('planing', '0007_prod_task_status'),
    ]

    operations = [
        migrations.CreateModel(
            name='Prod_tasks',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('qty_plain', models.PositiveIntegerField(verbose_name='Количество по плану')),
                ('create_date', models.DateTimeField(verbose_name='дата выдачи')),
                ('creator', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
                ('importances', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.Prod_task_importances')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
                ('status', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.OperationsPlanStatus')),
            ],
            options={
                'verbose_name': 'Задания в производство',
            },
        ),
    ]



import logging

from django.db import transaction

from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.progress import Progress
from kaf_pas.kd.models.pathes import Pathes

logger = logging.getLogger(__name__)


class UploadesQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class UploadesManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'lastmodified': record.lastmodified,
            'path_id': record.path.id,
            'absolute_path': record.path.absolute_path,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return UploadesQuerySet(self.model, using=self._db)

    def deleteFromRequest(self, request):
        from isc_common.auth.models.user import User

        from isc_common.management.commands.refresh_mat_views import refresh_mat_view
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.ckk.models.item_image_refs import Item_image_refs
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.documents import Documents
        from kaf_pas.kd.models.documents_history import Documents_history
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
        from kaf_pas.kd.models.lotsman_documents_hierarcy_refs import Lotsman_documents_hierarcy_refs
        from kaf_pas.kd.models.uploades_documents import Uploades_documents
        from kaf_pas.kd.models.uploades_log import Uploades_log
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail

        ids = request.GET.getlist('ids')
        user = User.objects.get(username=request.GET.get('ws_channel').split('_')[1])

        res = 0
        with transaction.atomic():
            for i in range(0, len(ids), 2):
                id = ids[i]
                visibleMode = ids[i + 1]

                if visibleMode != "none":
                    res += super().filter(id=id).soft_delete(visibleMode=visibleMode)
                else:
                    lotsman_cnt = 0

                    for upload_document in Uploades_documents.objects.filter(upload_id=id):
                        lotsman_cnt += Lotsman_documents_hierarcy.objects.filter(document=upload_document.document).count()

                    progress = Progress(
                        qty=Uploades_documents.objects.filter(upload_id=id).count() + lotsman_cnt,
                        id=id,
                        user=user,
                        message='Удаление закачек',
                        title='Выполнено'
                    )
                    try:
                        lotsman_cnt = 0
                        doc_cnt = 0

                        for upload_document in Uploades_documents.objects.filter(upload_id=id):

                            Document_attr_cross.objects.filter(document=upload_document.document).delete()

                            for item in Item.objects.filter(document=upload_document.document):
                                ItemManager.delete_recursive(item=item, delete_lines=True, user=user)

                                Operations_item.objects.filter(item=item).delete()

                                Ready_2_launch_detail.objects.filter(item=item).delete()
                                item.delete()

                            for documents_history in Documents_history.objects.filter(new_document=upload_document.document):
                                documents_history.old_document.props |= Documents.props.relevant
                                documents_history.old_document.save()
                                documents_history.delete()

                            for thumb in Documents_thumb.objects.filter(document=upload_document.document):
                                Item_image_refs.objects.filter(thumb=thumb).delete()
                                thumb.delete()

                            for thumb10 in Documents_thumb10.objects.filter(document=upload_document.document):
                                Item_image_refs.objects.filter(thumb10=thumb10).delete()
                                thumb10.delete()

                            for lotsman_document in Lotsman_documents_hierarcy.objects.filter(document=upload_document.document):
                                Lotsman_documents_hierarcy_refs.objects.filter(child=lotsman_document).delete()
                                Lotsman_documents_hierarcy_refs.objects.filter(parent=lotsman_document).delete()

                                for item in Item.objects.filter(lotsman_document=lotsman_document):
                                    ItemManager.delete_recursive(item=item, delete_lines=True, user=user)

                                    Operations_item.objects.filter(item=item).delete()

                                    Ready_2_launch_detail.objects.filter(item=item).delete()
                                    item.delete()

                                for thumb in Documents_thumb.objects.filter(lotsman_document=lotsman_document):
                                    Item_image_refs.objects.filter(thumb=thumb).delete()
                                    thumb.delete()

                                for thumb10 in Documents_thumb10.objects.filter(lotsman_document=lotsman_document):
                                    Item_image_refs.objects.filter(thumb10=thumb10).delete()
                                    thumb10.delete()

                                Lotsman_documents_hierarcy_files.objects.filter(lotsman_document=lotsman_document).delete()
                                lotsman_document.delete()
                                lotsman_cnt += 1
                                progress.step()

                            logger.debug(f'Deliting: {upload_document.document}')
                            upload_document.document.delete()
                            doc_cnt += 1
                            logger.debug('Done.')
                            progress.step()
                        Uploades_log.objects.filter(upload_id=id).delete()

                        if doc_cnt > 0:
                            progress.setContentsLabel('<h3>Refresh kd_documents_mview</h3>')
                            refresh_mat_view('kd_documents_mview')

                        if lotsman_cnt > 0:
                            progress.setContentsLabel('<h3>Refresh kd_lotsman_documents_hierarcy_mview</h3>')
                            refresh_mat_view('kd_lotsman_documents_hierarcy_mview')
                        progress.close()
                    except Exception as ex:
                        progress.close()
                        raise ex

                    res += super().filter(id=id).delete()[0]
        return res


class Uploades(AuditModel):
    path = ForeignKeyCascade(Pathes)
    objects = UploadesManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Загрузки внешних данных'
